<?php


class Jackmail_Campaign extends Jackmail_List_And_Campaign_Common_Core {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_campaign_page', array( $this, 'campaign_page_callback' ) );

			add_action( 'wp_ajax_jackmail_get_campaign', array( $this, 'get_campaign_callback' ) );

			add_action( 'wp_ajax_jackmail_get_fields_and_ids', array( $this, 'get_fields_and_ids_callback' ) );

			add_action( 'wp_ajax_jackmail_get_campaign_lists_available', array( $this, 'get_campaign_lists_available_callback' ) );

			add_action( 'wp_ajax_jackmail_get_campaign_contacts', array( $this, 'get_campaign_contacts_callback' ) );

			add_action( 'wp_ajax_jackmail_export_campaign_contacts', array( $this, 'export_campaign_contacts_callback' ) );

			add_action( 'wp_ajax_jackmail_set_campaign_lists', array( $this, 'set_campaign_lists_callback' ) );

			add_action( 'wp_ajax_jackmail_create_campaign', array( $this, 'create_campaign_callback' ) );

			add_action( 'wp_ajax_jackmail_update_campaign', array( $this, 'update_campaign_callback' ) );

			add_action( 'wp_ajax_jackmail_create_campaign_with_data', array( $this, 'create_campaign_with_data_callback' ) );

			add_action( 'wp_ajax_jackmail_campaign_last_step_checker', array( $this, 'campaign_last_step_checker_callback' ) );

			add_action( 'wp_ajax_jackmail_send_campaign_test', array( $this, 'send_campaign_test_callback' ) );

			add_action( 'wp_ajax_jackmail_send_campaign', array( $this, 'send_campaign_callback' ) );

		}

	}

	public function campaign_page_callback() {
		$this->core->check_auth();
		$params = array(
			'page_type'      => 'campaign',
			'campaign_type'  => 'campaign',
			'scenario_type'  => '',
			'display_emojis' => $this->display_emojis()
		);
		$this->core->include_html_file( 'campaign', $params );
		die;
	}

	public function get_campaign_callback() {
		$this->core->check_auth();
		$campaign = null;
		if ( isset( $_POST['id'] ) ) {
			$id_campaign = $this->core->request_text_data( $_POST['id'] );
			$campaign    = $this->get_campaign( $id_campaign );
		}
		wp_send_json( $campaign );
		die;
	}

	private function get_campaign( $id_campaign ) {
		global $wpdb;
		if ( $id_campaign === '0' ) {
			$link_tracking      = get_option( 'jackmail_link_tracking' );
			$content_email_json = get_option( 'jackmail_default_template' );
			if ( $content_email_json === false ) {
				$content_email_json = '';
			}
			if ( $this->core->emailbuilder_installed() === false ) {
				$content_email_json = '';
			}
			$campaign = array(
				'id'                         => '0',
				'id_lists'                   => '[]',
				'fields'                     => '[]',
				'name'                       => __( 'Campaign with no name', 'jackmail-newsletters' ),
				'object'                     => '',
				'sender_name'                => '',
				'sender_email'               => '',
				'reply_to_name'              => '',
				'reply_to_email'             => '',
				'nb_contacts'                => '0',
				'nb_contacts_valids'         => '0',
				'link_tracking'              => $link_tracking,
				'content_email_json'         => $content_email_json,
				'content_email_html'         => '',
				'content_email_txt'          => '',
				'status'                     => 'DRAFT',
				'content_size'               => true,
				'content_images_size'        => true,
				'send_option'                => 'NOW',
				'send_option_date_begin_gmt' => '0000-00-00 00:00:00',
				'send_option_date_end_gmt'   => '0000-00-00 00:00:00'
			);
		} else {
			$sql      = "
			SELECT *
			FROM `{$wpdb->prefix}jackmail_campaigns`
			WHERE `id` = %s
			AND `status` = 'DRAFT'";
			$campaign = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
			if ( isset( $campaign->id ) ) {
				$campaign->content_size        = $this->get_content_size( $campaign->content_email_json, $campaign->content_email_html, $campaign->content_email_txt );
				$campaign->content_images_size = $this->get_content_images_size( $campaign->content_email_images );
			}
		}
		return $campaign;
	}

	public function get_fields_and_ids_callback() {
		$this->core->check_auth();
		$json = array();
		if ( isset( $_POST['fields'] ) ) {
			$fields = $this->core->request_text_data( $_POST['fields'] );
			$fields = $this->core->explode_fields( $fields );
			if ( is_array( $fields ) ) {
				foreach ( $fields as $field ) {
					$field = $this->core->str_to_upper( $field );
					$id    = str_replace( '=', '', base64_encode( $field ) );
					if ( $field === 'EMAIL' ) {
						$id = 'email';
					}
					$json[] = array(
						'id'   => $id,
						'name' => $field
					);
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function get_campaign_lists_available_callback() {
		$this->core->check_auth();
		global $wpdb;
		$lists = array();
		if ( isset( $_POST['id'] ) ) {
			$id_campaign = $this->core->request_text_data( $_POST['id'] );
			$sql         = "
			SELECT `l`.`id`, `l`.`name`, `l`.`id_campaign`, `l`.`nb_contacts` AS `nb_display_contacts`, `l`.`type`
			FROM `{$wpdb->prefix}jackmail_lists` AS `l`
			ORDER BY `l`.`id` DESC";
			$lists       = $wpdb->get_results( $sql );
			foreach ( $lists as $key => $list ) {
				$lists[ $key ]->selected = false;
				$id_list                 = $list->id;
				if ( $this->is_plugin_special_list( $list->nb_display_contacts, $list->type ) ) {
					$plugin = $this->core->explode_data( $list->type );
					if ( isset( $plugin['name'] ) && isset( $plugin['id'] ) ) {
						$plugin_name                        = $plugin['name'];
						$plugin_id                          = $plugin['id'];
						$lists[ $key ]->nb_display_contacts = $this->get_plugin_list_nb_contacts( $plugin_name, $id_list, $plugin_id );
					}
				}
			}
			if ( $id_campaign !== '0' ) {
				$table_lists         = "{$wpdb->prefix}jackmail_campaigns";
				$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
				$sql                 = "SELECT `id_lists` FROM `{$table_lists}` WHERE `id` = %s";
				$campaign            = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
				if ( isset( $campaign->id_lists ) ) {
					$id_lists = $this->core->explode_data( $campaign->id_lists );
					foreach ( $id_lists as $id_list ) {
						$sql         = "SELECT COUNT( * ) FROM `{$table_list_contacts}` WHERE `id_list` = %s";
						$nb_contacts = $wpdb->get_var( $wpdb->prepare( $sql, $id_list ) );
						if ( $nb_contacts !== '0' ) {
							foreach ( $lists as $key => $list ) {
								if ( $list->id === $id_list ) {
									$lists[ $key ]->nb_display_contacts = $nb_contacts;
									$lists[ $key ]->selected            = true;
								}
							}
						}
					}
				}
			}
		}
		wp_send_json( $lists );
		die;
	}

	public function get_campaign_contacts_callback() {
		$this->core->check_auth();
		$json = $this->get_or_export_campaign_contacts( (string) $this->core->grid_limit() );
		wp_send_json( $json );
		die;
	}

	public function export_campaign_contacts_callback() {
		$this->core->check_auth();
		$json = $this->get_or_export_campaign_contacts( (string) $this->core->export_send_limit() );
		wp_send_json( $json );
		die;
	}

	private function get_or_export_campaign_contacts( $part ) {
		$data = $this->get_list_or_get_campaign_contacts_post_data();
		if ( isset( $data ['id'] ) && isset( $data ['begin'] ) && isset( $data ['sort_by'] ) && isset( $data ['sort_order'] ) && isset( $data ['search'] ) && isset( $data ['targeting_rules'] ) ) {
			if ( $data ['id'] === '0' ) {
				$json = array(
					'list'               => array(
						'id'     => '',
						'fields' => '[]'
					),
					'contacts'           => array(),
					'nb_contacts'        => '0',
					'nb_contacts_search' => '0'
				);
			} else {
				if ( $part === (string) $this->core->grid_limit() ) {
					$this->update_campaign_contacts_blacklists( $data ['id'] );
				}
				$json = $this->get_campaign_contacts( $data ['id'], $data ['begin'], $part, $data ['sort_by'], $data ['sort_order'], $data['search'], $data ['targeting_rules'] );
			}
			return $json;
		}
		return array();
	}

	private function update_campaign_contacts_blacklists( $id_campaign ) {
		global $wpdb;
		$table_campaign_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
		if ( $this->core->check_table_exists( $table_campaign_list_contacts, false ) ) {
			$sql      = "
			SELECT `id_lists`
			FROM `{$wpdb->prefix}jackmail_campaigns`
			WHERE `id` = %s
			AND `status` = 'DRAFT'";
			$campaign = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
			if ( isset( $campaign->id_lists ) ) {
				$id_lists = $this->core->explode_data( $campaign->id_lists );
				foreach ( $id_lists as $key => $id_list ) {
					$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}";
					if ( $this->core->check_table_exists( $table_list_contacts, false ) ) {
						$sql = "
						UPDATE `{$table_campaign_list_contacts}` AS `cl`
						INNER JOIN `{$table_list_contacts}` AS `l` ON `l`.`email` = `cl`.`email` AND `l`.`blacklist` != `cl`.`blacklist`
						SET `cl`.`blacklist` = `l`.`blacklist`
						WHERE `cl`.`id_list` = %s";
						$wpdb->query( $wpdb->prepare( $sql, $id_list ) );
					}
				}
			}
		}
	}

	public function set_campaign_lists_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( isset( $_POST['id_campaign'] ) && isset( $_POST['id_lists'] ) ) {
			$id_campaign         = $this->core->request_text_data( $_POST['id_campaign'] );
			$id_lists            = $this->core->request_text_data( $_POST['id_lists'] );
			$id_lists            = $this->core->explode_data( $id_lists );
			$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
			$sql                 = "
			SELECT `id_lists`, `fields`
			FROM `{$wpdb->prefix}jackmail_campaigns`
			WHERE `id` = %s";
			$current_campaign    = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
			if ( isset( $current_campaign->id_lists ) && isset( $current_campaign->fields ) ) {
				$all_id_lists     = $this->core->explode_data( $current_campaign->id_lists );
				$new_fields       = $this->core->explode_fields( $current_campaign->fields );
				$deleted_id_lists = array_diff( $all_id_lists, $id_lists );
				foreach ( $deleted_id_lists as $id_list ) {
					$this->core->delete_campaign_list_contact( $id_campaign, array(
						'id_list' => $id_list
					) );
				}
				$new_id_lists = array();
				$this->actualize_plugins_lists();
				foreach ( $id_lists as $id_list ) {
					if ( ! in_array( $id_list, $all_id_lists ) ) {
						$sql           = "
						SELECT `fields`, `type`, `nb_contacts`
						FROM `{$wpdb->prefix}jackmail_lists`
						WHERE `id` = %s";
						$list          = $wpdb->get_row( $wpdb->prepare( $sql, $id_list ) );
						$list_found    = '';
						$column_fields = '';
						$sql_rows      = '';
						if ( isset( $list->type ) ) {
							$column_fields = $list->fields;
							$list_found    = 'normal';
							if ( $this->is_plugin_special_list( $list->nb_contacts, $list->type ) ) {
								$plugin = $this->core->explode_data( $list->type );
								if ( isset( $plugin['name'] ) && isset( $plugin['id'] ) ) {
									$plugin_name = $plugin['name'];
									$plugin_id   = $plugin['id'];
									$list_found  = $plugin_name;
									$sql_request = $this->get_plugin_list_sql( $plugin_name, 'ALL', '', $plugin_id, $id_list, 'email', 'ASC', '' );
									if ( isset( $sql_request['rows_for_insert'] ) && isset( $sql_request['fields'] ) ) {
										$sql_rows      = $sql_request['rows_for_insert'];
										$column_fields = $sql_request['fields'];
									}
								}
							}
						}
						if ( $list_found !== '' ) {
							$select_fields          = array();
							$select_fields[]        = '`email`';
							$select_fields[]        = '`blacklist`';
							$select_fields_insert   = array();
							$select_fields_insert[] = '`email`';
							$select_fields_insert[] = '`blacklist`';
							$columns                = $this->core->get_table_columns( 'jackmail_campaigns_lists_contacts_' . $id_campaign );
							if ( $column_fields !== '' ) {
								$fields_array = $this->core->explode_fields( $column_fields );
								foreach ( $fields_array as $i => $value ) {
									$nb_fields = count( $new_fields );
									$field_id  = array_search( $value, $new_fields );
									if ( $field_id === false ) {
										$field = 'field' . ( $nb_fields + 1 );
									} else {
										$field = 'field' . ( $field_id + 1 );
									}
									if ( $field_id === false ) {
										$new_fields[] = $value;
										if ( ! in_array( $field, $columns ) ) {
											$this->core->create_list_field( $table_list_contacts, $field );
											$columns = $this->core->get_table_columns( 'jackmail_campaigns_lists_contacts_' . $id_campaign );
										}
									}
									$field_select           = 'field' . ( $i + 1 );
									$select_fields[]        = '`' . $field . '`';
									$select_fields_insert[] = '`' . $field_select . '` AS `' . $field . '`';
								}
							}
							$select_fields_implode = implode( ', ', $select_fields );
							if ( count( $select_fields ) === count( $select_fields_insert ) ) {
								$duplicate_key_update = array();
								foreach ( $columns as $column ) {
									if ( $column !== 'email' && $column !== 'blacklist' ) {
										if ( in_array( '`' . $column . '`', $select_fields ) ) {
											$duplicate_key_update[] = "`{$column}` = IF( `{$table_list_contacts}`.`blacklist` < VALUES(`blacklist`), VALUES(`{$column}`), `{$table_list_contacts}`.`{$column}` )";
										} else if ( $column === 'id_list' ) {
											$duplicate_key_update[] = "`id_list` = IF( `{$table_list_contacts}`.`blacklist` < VALUES(`blacklist`), VALUES(`id_list`), `{$table_list_contacts}`.`id_list` )";
										} else {
											$duplicate_key_update[] = "`{$column}` = IF( `{$table_list_contacts}`.`blacklist` < VALUES(`blacklist`), '', `{$table_list_contacts}`.`{$column}` )";
										}
									}
								}
								$duplicate_key_update[] = "`blacklist` = IF( `{$table_list_contacts}`.`blacklist` < VALUES(`blacklist`), VALUES(`blacklist`), `{$table_list_contacts}`.`blacklist` )";
								$duplicate_key_update   = implode( ', ', $duplicate_key_update );
								if ( $list_found === 'normal' ) {
									if ( $this->core->check_table_exists( 'jackmail_lists_contacts_' . $id_list ) ) {
										if ( isset( $_POST['search'] ) && isset( $_POST['targeting_rules'] ) && isset( $_POST['contacts_selection'] ) && isset( $_POST['contacts_selection_type'] ) ) {
											$search                  = $this->core->request_text_data( $_POST['search'] );
											$targeting_rules         = $this->core->request_text_data( $_POST['targeting_rules'] );
											$contacts_selection      = $this->core->request_text_data( $_POST['contacts_selection'] );
											$contacts_selection_type = $this->core->request_text_data( $_POST['contacts_selection_type'] );
											$table_list_from         = "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}";
											$contacts_sql            = $this->get_list_or_get_campaign_contacts_sql( $table_list_from, 'ALL_FILTERED', '', '', '', $search, $targeting_rules, $contacts_selection, $contacts_selection_type );
											if ( isset( $contacts_sql['begin'] ) && isset( $contacts_sql['sql_contacts'] ) && isset( $contacts_sql['sql_nb_contacts'] ) && isset( $contacts_sql['sql_nb_contacts_search'] ) && isset( $contacts_sql['sql_values'] ) && is_array( $contacts_sql['sql_values'] ) ) {
												$sql_contacts  = $contacts_sql['sql_contacts'];
												$sql_values    = $contacts_sql['sql_values'];
												$select_fields = '(' . implode( ', ', $select_fields ) . ')';
												$wpdb->query( "TRUNCATE `{$table_list_contacts}`" );
												if ( count( $sql_values ) === 0 ) {
													$sql = "INSERT INTO `{$table_list_contacts}` {$select_fields} {$sql_contacts}";
													$wpdb->query( $sql );
												} else {
													$sql = "INSERT INTO `{$table_list_contacts}` {$select_fields} {$sql_contacts}";
													$wpdb->query( $wpdb->prepare( $sql, $sql_values ) );
												}
												$this->core->update_list_contact_or_campaign_list_contact( $table_list_contacts, array(
													'id_list' => $id_list
												), array(
													'id_list' => '0'
												) );
												$update = array(
													'id_lists' => json_encode( array( $id_list ) ),
													'fields'   => $column_fields
												);
												$this->core->updated_campaign_list_contact( $id_campaign, $update );
											}
										} else {
											$select_fields_insert_implode = implode( ', ', $select_fields_insert );
											$sql                          = "
											INSERT INTO `{$table_list_contacts}` ( {$select_fields_implode}, `id_list` ) ( SELECT {$select_fields_insert_implode}, %s AS `id_list` FROM `{$wpdb->prefix}jackmail_lists_contacts_{$id_list}` )
											ON DUPLICATE KEY UPDATE {$duplicate_key_update}";
											$wpdb->query( $wpdb->prepare( $sql, $id_list ) );
										}
									}
								} else {
									if ( $sql_rows !== '' ) {
										foreach ( $select_fields_insert as $select_field_insert ) {
											$select_field_insert = explode( ' AS ', $select_field_insert );
											if ( count( $select_field_insert ) === 2 ) {
												$field    = $select_field_insert [0];
												$field_as = $select_field_insert [1];
												$sql_rows = str_replace( 'AS ' . $field, 'AS ' . $field_as, $sql_rows );
											}
										}
										$sql = "
										INSERT INTO `{$table_list_contacts}` ( {$select_fields_implode}, `id_list` ) ( {$sql_rows} )
										ON DUPLICATE KEY UPDATE {$duplicate_key_update}";
										$wpdb->query( $sql );
									}
								}
								$new_id_lists[] = $id_list;
							}
						}
					} else {
						$new_id_lists[] = $id_list;
					}
				}
				$new_id_lists = array_unique( $new_id_lists );
				$update       = array(
					'id_lists' => $this->core->implode_data( $new_id_lists ),
					'fields'   => $this->core->implode_fields( $new_fields )
				);
				$this->core->updated_campaign_list_contact( $id_campaign, $update );
			}
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function create_campaign_callback() {
		$this->core->check_auth();
		$result = array(
			'success'             => false,
			'id'                  => '0',
			'content_size'        => false,
			'content_images_size' => false,
			'content_email_json'  => '',
			'content_email_html'  => '',
			'content_email_txt'   => ''
		);
		if ( isset( $_POST['name'] ) && isset( $_POST['object'] ) && isset( $_POST['sender_name'] ) && isset( $_POST['sender_email'] ) && isset( $_POST['reply_to_name'] ) && isset( $_POST['reply_to_email'] ) && isset( $_POST['content_email_json'] ) && isset( $_POST['content_email_html'] ) && isset( $_POST['content_email_txt'] ) && isset( $_POST['send_option'] ) && isset( $_POST['send_option_date_begin_gmt'] ) && isset( $_POST['send_option_date_end_gmt'] ) ) {
			$name           = $this->core->request_text_data( $_POST['name'] );
			$object         = $this->core->request_text_data( $_POST['object'] );
			$sender_name    = $this->core->request_text_data( $_POST['sender_name'] );
			$sender_email   = $this->core->request_text_data( $_POST['sender_email'] );
			$reply_to_name  = $this->core->request_text_data( $_POST['reply_to_name'] );
			$reply_to_email = $this->core->request_text_data( $_POST['reply_to_email'] );
			if ( isset( $_POST['link_tracking'] ) ) {
				$link_tracking = $this->core->request_text_data( $_POST['link_tracking'] );
			} else {
				$link_tracking = get_option( 'jackmail_link_tracking' );
			}
			$content_email_json         = $this->core->request_stripslashes( $_POST['content_email_json'] );
			$content_email_html         = $this->core->request_stripslashes( $_POST['content_email_html'] );
			$content_email_txt          = $this->core->request_textarea_data( $_POST['content_email_txt'] );
			$send_option                = $this->core->request_text_data( $_POST['send_option'] );
			$send_option_date_begin_gmt = '0000-00-00 00:00:00';
			$send_option_date_end_gmt   = '0000-00-00 00:00:00';
			if ( $send_option !== 'NOW' ) {
				$send_option_date_begin_gmt = $this->core->request_text_data( $_POST['send_option_date_begin_gmt'] );
				$send_option_date_end_gmt   = $this->core->request_text_data( $_POST['send_option_date_end_gmt'] );
			}
			$preview       = $this->core->generate_jackmail_preview_filename();
			$content_email = $this->core->set_content_email( 'campaign', '0', $preview, $content_email_json, $content_email_html, $content_email_txt );
			if ( $content_email !== false ) {
				if ( isset( $content_email['content_email_json'] ) && isset( $content_email['content_email_html'] ) && isset( $content_email['content_email_txt'] ) && isset( $content_email['content_email_images'] ) ) {
					$content_email_json   = $content_email['content_email_json'];
					$content_email_html   = $content_email['content_email_html'];
					$content_email_txt    = $content_email['content_email_txt'];
					$content_email_images = $content_email['content_email_images'];
					$current_date_gmt     = $this->core->get_current_time_gmt_sql();
					$id_campaign          = $this->core->insert_campaign( array(
						'id_lists'                   => '[]',
						'fields'                     => '[]',
						'name'                       => $name,
						'object'                     => $object,
						'sender_name'                => $sender_name,
						'sender_email'               => $sender_email,
						'reply_to_name'              => $reply_to_name,
						'reply_to_email'             => $reply_to_email,
						'link_tracking'              => $link_tracking,
						'content_email_json'         => $content_email_json,
						'content_email_html'         => $content_email_html,
						'content_email_txt'          => $content_email_txt,
						'content_email_images'       => $content_email_images,
						'preview'                    => $preview,
						'created_date_gmt'           => $current_date_gmt,
						'updated_date_gmt'           => $current_date_gmt,
						'updated_by'                 => get_current_user_id(),
						'status'                     => 'DRAFT',
						'send_option'                => $send_option,
						'send_option_date_begin_gmt' => $send_option_date_begin_gmt,
						'send_option_date_end_gmt'   => $send_option_date_end_gmt
					) );
					if ( $id_campaign !== false && $id_campaign !== 0 && $id_campaign !== '0' ) {
						if ( $this->core->create_campaign_list_table( $id_campaign ) !== false ) {
							$result = array(
								'success'             => true,
								'id'                  => $id_campaign,
								'content_size'        => $this->get_content_size( $content_email_json, $content_email_html, $content_email_txt ),
								'content_images_size' => $this->get_content_images_size( $content_email_images ),
								'content_email_json'  => $content_email_json,
								'content_email_html'  => $content_email_html,
								'content_email_txt'   => $content_email_txt
							);
						}
					}
				}
			}
		}
		wp_send_json( $result );
		die;
	}

	public function update_campaign_callback() {
		$this->core->check_auth();
		global $wpdb;
		$result = array(
			'success'             => false,
			'content_size'        => false,
			'content_images_size' => false,
			'content_email_json'  => '',
			'content_email_html'  => '',
			'content_email_txt'   => ''
		);
		if ( isset( $_POST['id'] ) && isset( $_POST['name'] ) && isset( $_POST['object'] ) && isset( $_POST['sender_name'] ) && isset( $_POST['sender_email'] ) && isset( $_POST['reply_to_name'] ) && isset( $_POST['reply_to_email'] ) && isset( $_POST['link_tracking'] ) && isset( $_POST['content_email_json'] ) && isset( $_POST['content_email_html'] ) && isset( $_POST['content_email_txt'] ) && isset( $_POST['send_option'] ) && isset( $_POST['send_option_date_begin_gmt'] ) && isset( $_POST['send_option_date_end_gmt'] ) ) {
			$id_campaign                = $this->core->request_text_data( $_POST['id'] );
			$name                       = $this->core->request_text_data( $_POST['name'] );
			$object                     = $this->core->request_text_data( $_POST['object'] );
			$sender_name                = $this->core->request_text_data( $_POST['sender_name'] );
			$sender_email               = $this->core->request_text_data( $_POST['sender_email'] );
			$reply_to_name              = $this->core->request_text_data( $_POST['reply_to_name'] );
			$reply_to_email             = $this->core->request_text_data( $_POST['reply_to_email'] );
			$link_tracking              = $this->core->request_text_data( $_POST['link_tracking'] );
			$content_email_json         = $this->core->request_stripslashes( $_POST['content_email_json'] );
			$content_email_html         = $this->core->request_stripslashes( $_POST['content_email_html'] );
			$content_email_txt          = $this->core->request_textarea_data( $_POST['content_email_txt'] );
			$send_option                = $this->core->request_text_data( $_POST['send_option'] );
			$send_option_date_begin_gmt = '0000-00-00 00:00:00';
			$send_option_date_end_gmt   = '0000-00-00 00:00:00';
			if ( $send_option !== 'NOW' ) {
				$send_option_date_begin_gmt = $this->core->request_text_data( $_POST['send_option_date_begin_gmt'] );
				$send_option_date_end_gmt   = $this->core->request_text_data( $_POST['send_option_date_end_gmt'] );
			}
			$current_date_gmt    = $this->core->get_current_time_gmt_sql();
			$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
			if ( $this->core->check_table_exists( $table_list_contacts, false ) ) {
				$sql      = "
				SELECT `preview`
				FROM `{$wpdb->prefix}jackmail_campaigns`
				WHERE `id` = %s";
				$campaign = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
				if ( isset( $campaign->preview ) ) {
					$content_email = $this->core->set_content_email( 'campaign', $id_campaign, $campaign->preview, $content_email_json, $content_email_html, $content_email_txt );
					if ( $content_email !== false ) {
						if ( isset( $content_email['content_email_json'] ) && isset( $content_email['content_email_html'] ) && isset( $content_email['content_email_txt'] ) && isset( $content_email['content_email_images'] ) ) {
							$content_email_json   = $content_email['content_email_json'];
							$content_email_html   = $content_email['content_email_html'];
							$content_email_txt    = $content_email['content_email_txt'];
							$content_email_images = $content_email['content_email_images'];
							$update_return        = $this->core->update_campaign( array(
								'name'                       => $name,
								'object'                     => $object,
								'sender_name'                => $sender_name,
								'sender_email'               => $sender_email,
								'reply_to_name'              => $reply_to_name,
								'reply_to_email'             => $reply_to_email,
								'link_tracking'              => $link_tracking,
								'content_email_json'         => $content_email_json,
								'content_email_html'         => $content_email_html,
								'content_email_txt'          => $content_email_txt,
								'content_email_images'       => $content_email_images,
								'send_option'                => $send_option,
								'send_option_date_begin_gmt' => $send_option_date_begin_gmt,
								'send_option_date_end_gmt'   => $send_option_date_end_gmt,
								'updated_date_gmt'           => $current_date_gmt,
								'updated_by'                 => get_current_user_id()
							), array(
								'id'     => $id_campaign,
								'status' => 'DRAFT'
							) );
							if ( $update_return !== false ) {
								$result = array(
									'success'             => true,
									'content_size'        => $this->get_content_size( $content_email_json, $content_email_html, $content_email_txt ),
									'content_images_size' => $this->get_content_images_size( $content_email_images ),
									'content_email_json'  => $content_email_json,
									'content_email_html'  => $content_email_html,
									'content_email_txt'   => $content_email_txt
								);
							}
						}
					}
				}
			}
		}
		wp_send_json( $result );
		die;
	}

	public function create_campaign_with_data_callback() {
		$this->core->check_auth();
		$result = array(
			'success' => false,
			'id'      => '0'
		);
		if ( isset( $_POST['sender_name'] ) && isset( $_POST['sender_email'] ) ) {
			$sender_name  = $this->core->request_text_data( $_POST['sender_name'] );
			$sender_email = $this->core->request_text_data( $_POST['sender_email'] );
			$campaign     = $this->get_campaign( '0' );
			$preview      = $this->core->generate_jackmail_preview_filename();
			if ( isset( $campaign['content_email_json'] ) && isset( $campaign['content_email_html'] ) && isset( $campaign['content_email_txt'] ) ) {
				$content_email = $this->core->set_content_email( 'campaign', '0', $preview, $campaign['content_email_json'], $campaign['content_email_html'], $campaign['content_email_txt'] );
				if ( $content_email !== false ) {
					if ( isset( $content_email['content_email_json'] ) && isset( $content_email['content_email_html'] ) && isset( $content_email['content_email_txt'] ) && isset( $content_email['content_email_images'] ) ) {
						$current_date_gmt = $this->core->get_current_time_gmt_sql();
						$id_campaign      = $this->core->insert_campaign( array(
							'id_lists'                   => '[]',
							'fields'                     => '[]',
							'name'                       => $campaign['name'],
							'object'                     => $campaign['object'],
							'sender_name'                => $sender_name,
							'sender_email'               => $sender_email,
							'reply_to_name'              => $campaign['reply_to_name'],
							'reply_to_email'             => $campaign['reply_to_email'],
							'link_tracking'              => $campaign['link_tracking'],
							'content_email_json'         => $content_email['content_email_json'],
							'content_email_html'         => $content_email['content_email_html'],
							'content_email_txt'          => $content_email['content_email_txt'],
							'content_email_images'       => $content_email['content_email_images'],
							'preview'                    => $preview,
							'created_date_gmt'           => $current_date_gmt,
							'updated_date_gmt'           => $current_date_gmt,
							'updated_by'                 => get_current_user_id(),
							'status'                     => 'DRAFT',
							'send_option'                => $campaign['send_option'],
							'send_option_date_begin_gmt' => $campaign['send_option_date_begin_gmt'],
							'send_option_date_end_gmt'   => $campaign['send_option_date_end_gmt']
						) );
						if ( $id_campaign !== false && $id_campaign !== 0 && $id_campaign !== '0' ) {
							if ( $this->core->create_campaign_list_table( $id_campaign ) !== false ) {
								$result = array(
									'success' => true,
									'id'      => $id_campaign
								);
							}
						}
					}
				}
			}
		}
		wp_send_json( $result );
		die;
	}

	public function campaign_last_step_checker_callback() {
		$this->core->check_auth();
		global $wpdb;
		$json = array(
			'nb_contacts_valids' => '0',
			'nb_credits_before'  => '0',
			'nb_credits_after'   => '0',
			'nb_credits_checked' => false,
			'subscription_type'  => '',
			'domain_is_valid'    => $this->core->custom_domain_is_valid(),
			'analysis_checked'   => false,
			'analysis'           => array()
		);
		if ( isset( $_POST['id'] ) ) {
			$id_campaign                  = $this->core->request_text_data( $_POST['id'] );
			$table_campaign_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
			if ( $this->core->check_table_exists( $table_campaign_list_contacts, false ) ) {
				$this->update_campaign_contacts_blacklists( $id_campaign );
				$sql      = "
				SELECT COUNT( * ) AS `nb_contacts_valids`
				FROM `{$table_campaign_list_contacts}`
				WHERE `blacklist` = '0'";
				$campaign = $wpdb->get_row( $sql );
				if ( isset( $campaign->nb_contacts_valids ) ) {
					$json['nb_contacts_valids'] = $campaign->nb_contacts_valids;
					$credits_available          = $this->core->get_credits_available( true );
					if ( $credits_available !== false ) {
						$json['nb_credits_before']  = (string) $credits_available['nb_credits'];
						$json['nb_credits_after']   = (string) ( $credits_available['nb_credits'] - $json['nb_contacts_valids'] );
						$json['nb_credits_checked'] = true;
						$json['subscription_type']  = $credits_available['subscription_type'];
					}
				}
				$sql  = "
				SELECT *
				FROM `{$wpdb->prefix}jackmail_campaigns`
				WHERE `id` = %s
				AND `status` = 'DRAFT'";
				$data = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
				if ( isset( $data->id ) ) {
					$fields                   = $data->fields;
					$object                   = $data->object;
					$sender_name              = $data->sender_name;
					$sender_email             = $data->sender_email;
					$reply_to_name            = $data->reply_to_name;
					$reply_to_email           = $data->reply_to_email;
					$content_email_json       = $data->content_email_json;
					$content_email_html       = $data->content_email_html;
					$content_email_txt        = $data->content_email_txt;
					$content_email_images     = $data->content_email_images;
					$link_tracking            = $data->link_tracking;
					$campaign_analysis_result = $this->get_campaign_analysis( $object, $sender_name, $sender_email, $reply_to_name, $reply_to_email, $content_email_json, $content_email_html, $content_email_txt, $content_email_images, $link_tracking, $fields );
					if ( $campaign_analysis_result !== false ) {
						$json['analysis_checked'] = true;
						$json['analysis']         = $campaign_analysis_result;
					}
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function send_campaign_test_callback() {
		$this->core->check_auth();
		global $wpdb;
		$result = array(
			'success' => false,
			'message' => 'ERROR'
		);
		if ( isset( $_POST['id'] ) && isset( $_POST['test_recipient'] ) ) {
			$id_campaign    = $this->core->request_text_data( $_POST['id'] );
			$test_recipient = $this->core->request_text_data( $_POST['test_recipient'] );
			$sql            = "
			SELECT *
			FROM `{$wpdb->prefix}jackmail_campaigns`
			WHERE `id` = %s
			AND `status` = 'DRAFT'";
			$data           = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
			if ( isset( $data->id ) ) {
				$campaign_id                = $data->campaign_id;
				$fields                     = $data->fields;
				$object                     = $data->object;
				$sender_name                = $data->sender_name;
				$sender_email               = $data->sender_email;
				$reply_to_name              = $data->reply_to_name;
				$reply_to_email             = $data->reply_to_email;
				$content_email_json         = $data->content_email_json;
				$content_email_html         = $data->content_email_html;
				$content_email_txt          = $data->content_email_txt;
				$content_email_images       = $data->content_email_images;
				$link_tracking              = $data->link_tracking;
				$send_option                = 'NOW';
				$send_option_date_begin_gmt = '';
				$send_option_date_end_gmt   = '';
				if ( is_email( $test_recipient ) ) {
					$campaign_result = $this->generate_campaign( $id_campaign, 'campaign', $campaign_id, '', $object, $sender_name, $sender_email, $reply_to_name, $reply_to_email, $content_email_json, $content_email_html, $content_email_txt, $content_email_images, array(), $link_tracking, $send_option, $send_option_date_begin_gmt, $send_option_date_end_gmt, $fields, '1', $test_recipient );
					$message         = $campaign_result['message'];
					$result          = array(
						'success' => $message === 'OK' ? true : false,
						'message' => $message
					);
				}
			}
		}
		wp_send_json( $result );
		die;
	}

	public function send_campaign_callback() {
		$this->core->check_auth();
		ini_set( 'max_execution_time', 600 );
		global $wpdb;
		$result = array(
			'success' => false,
			'message' => 'ERROR'
		);
		if ( isset( $_POST['id'] ) && isset( $_POST['send_option'] ) && isset( $_POST['send_option_date_begin_gmt'] ) && isset( $_POST['send_option_date_end_gmt'] ) && isset( $_POST['nb_contacts_valids_displayed'] ) ) {
			$id_campaign = $this->core->request_text_data( $_POST['id'] );
			$send_option = $this->core->request_text_data( $_POST['send_option'] );
			if ( $send_option === 'NOW' ) {
				$send_option_date_begin_gmt = '0000-00-00 00:00:00';
				$send_option_date_end_gmt   = '0000-00-00 00:00:00';
			} else {
				$send_option_date_begin_gmt = $this->core->request_text_data( $_POST['send_option_date_begin_gmt'] );
				$send_option_date_end_gmt   = $this->core->request_text_data( $_POST['send_option_date_end_gmt'] );
			}
			$nb_contacts_valids_displayed = $this->core->request_text_data( $_POST['nb_contacts_valids_displayed'] );
			$current_date_gmt             = $this->core->get_current_time_gmt_sql();
			$update_return                = $this->core->update_campaign( array(
				'send_option'                => $send_option,
				'send_option_date_begin_gmt' => $send_option_date_begin_gmt,
				'send_option_date_end_gmt'   => $send_option_date_end_gmt,
				'updated_date_gmt'           => $current_date_gmt
			), array(
				'id'     => $id_campaign,
				'status' => 'DRAFT'
			) );
			if ( $update_return !== false ) {
				$sql  = "
				SELECT *
				FROM `{$wpdb->prefix}jackmail_campaigns`
				WHERE `id` = %s
				AND `status` = 'DRAFT'";
				$data = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
				if ( isset( $data->id ) ) {
					$campaign_id                = $data->campaign_id;
					$fields                     = $data->fields;
					$object                     = $data->object;
					$sender_name                = $data->sender_name;
					$sender_email               = $data->sender_email;
					$reply_to_name              = $data->reply_to_name;
					$reply_to_email             = $data->reply_to_email;
					$content_email_json         = $data->content_email_json;
					$content_email_html         = $data->content_email_html;
					$content_email_txt          = $data->content_email_txt;
					$content_email_images       = $data->content_email_images;
					$link_tracking              = $data->link_tracking;
					$send_option                = $data->send_option;
					$send_option_date_begin_gmt = $data->send_option_date_begin_gmt;
					$send_option_date_end_gmt   = $data->send_option_date_end_gmt;
					$nb_contacts_valids         = $data->nb_contacts_valids;
					if ( $nb_contacts_valids <= $nb_contacts_valids_displayed ) {
						$campaign_result = $this->generate_campaign( $id_campaign, 'campaign', $campaign_id, '', $object, $sender_name, $sender_email, $reply_to_name, $reply_to_email, $content_email_json, $content_email_html, $content_email_txt, $content_email_images, array(), $link_tracking, $send_option, $send_option_date_begin_gmt, $send_option_date_end_gmt, $fields, $nb_contacts_valids_displayed );
						$message         = $campaign_result['message'];
						if ( $message === 'OK' ) {
							$result = array(
								'success' => true,
								'message' => $message
							);
						} else {
							$this->core->update_campaign( array(
								'status' => 'DRAFT'
							), array(
								'id' => $id_campaign
							) );
							$result = array(
								'success' => false,
								'message' => $message
							);
						}
					} else {
						$result = array(
							'success' => false,
							'message' => 'NB_DISPLAYED_CONTACTS'
						);
					}
				}
			}
		}
		wp_send_json( $result );
		die;
	}

}