'use strict';

angular.module( 'jackmail.services' ).factory( 'CampaignService', [
	'$rootScope',
	function( $rootScope ) {

		var service = {

			get_send_campaign_message: function( message ) {
				var error = $rootScope.translations.an_error_occurred;
				if ( message === 'ERROR_READ_EMAIL_CONTENT_FILE' ) {
					error = $rootScope.translations.an_error_occurred_the_campaign_was_not_found;
				}
				else if ( message === 'FORBIDDEN' ) {
					error = $rootScope.translations.the_user_name_or_the_password_is_incorrect;
				}
				else if ( message === 'MESSAGE_CONTENT_IS_TOO_LARGE' ) {
					error = $rootScope.translations.error_content_is_too_long;
				}
				else if ( message === 'MESSAGE_IMAGES_SIZE_IS_TOO_LARGE' ) {
					error = $rootScope.translations.error_images_are_too_large;
				}
				else if ( message === 'MISSING_PHP_OPENSSL' ) {
					error = $rootScope.translations.please_activate_the_extension_openssl_random_pseudo_bytes_on_your_web_server;
				}
				else if ( message === 'MISSING_GZDECODE_OR_GZENCODE' ) {
					error = $rootScope.translations.gzdecode_or_gzencode_php_function_not_found;
				}
				else if ( message === 'MISSING_BASE64_ENCODE_OR_BASE64_DECODE' ) {
					error = $rootScope.translations.base64_encode_or_base64_decode_php_function_not_found;
				}
				else if ( message === 'MISSING_JSON_ENCODE_OR_JSON_DECODE' ) {
					error = $rootScope.translations.json_encode_or_json_decode_php_function_not_found;
				}
				else if ( message === 'ERROR_WHILE_LOADING_IMAGES' ) {
					error = $rootScope.translations.error_while_loading_images;
				}
				else if ( message === 'ERROR_WHILE_LOADING_CONTENT' ) {
					error = $rootScope.translations.error_while_loading_content;
				}
				else if ( message === 'NOT_ENOUGH_CREDITS' ) {
					error = $rootScope.translations.not_enough_credits;
				}
				else if ( message === 'ERROR_WHILE_CHECKING_CREDITS' ) {
					error = $rootScope.translations.error_while_checking_credits_available;
				}
				else if ( message === 'NO_VALIDS_RECIPIENTS' ) {
					error = $rootScope.translations.no_valids_recipients;
				}
				else if ( message === 'NB_DISPLAYED_CONTACTS' ) {
					error = $rootScope.translations.data_displayed_are_not_up_to_date_click_ok_to_reload_it_before_you_send_the_campaign;
					if ( vm.c_common.campaign.send_option !== 'NOW' && !vm.c_common.sending_campaign ) {
						error = $rootScope.translations.data_displayed_are_not_up_to_date_click_ok_to_reload_it_before_you_schedule_the_campaign;
					}
				}
				else {
					error = $rootScope.translations.an_error_occurred + ' (' + message + ')';
				}
				return error;
			}

		};

		return service;

	} ] );

angular.module( 'jackmail.services' ).factory( 'EmailContentService', [
	'$rootScope', '$timeout', '$document', '$window', 'UrlService', '$http', '$q',
	function( $rootScope, $timeout, $document, $window, UrlService, $http, $q ) {

		var refresh_email_content_height = false;

		var current_mode = '';

		var editor;

		var content_is_loading = false;

		function display_emailbuilder() {
			if ( $rootScope.settings.emailbuilder_installed ) {
				$rootScope.show_emailbuilder = true;
				refresh_email_content_height = true;
				$timeout( function() {
					service.refresh_email_content_height();
				} );
			}
		}

		function display_email_content_editor( current_content_email_type ) {
			$rootScope.show_emailbuilder = false;
			refresh_email_content_height = true;
			init_editor_if_needed( current_content_email_type );
			refresh_html_editor();
			$timeout( function() {
				service.refresh_email_content_height();
			} );
		}

		function init_editor_if_needed( current_content_email_type ) {
			var mode = 'text/html';
			var theme = 'monokai';
			if ( current_content_email_type === 'txt' ) {
				mode = 'text/plain';
				theme = '';
			}
			if ( editor === undefined ) {
				editor = new SbEditor( document.getElementById( 'jackmail_content_email' ), {
					'mode': mode,
					'theme': theme,
					'lineNumbers': true
				} );
			}
			else {
				if ( mode !== current_mode ) {
					editor.theme = theme;
					editor.mode = mode;
					refresh_html_editor();
				}
			}
			current_mode = mode;
		}

		function refresh_html_editor() {
			$timeout( function() {
				editor.refresh();
				if ( editor.content === '' ) {
					editor.focus();
				}
			}, 100 );
		}

		function get_emailbuilder_json() {
			if ( $rootScope.settings.emailbuilder_installed ) {
				try {
					var json = content_is_loading;
					if ( json === false ) {
						json = window.angularComponentRef.zone.run( function() {
							return window.angularComponentRef.component.saveContent();
						} );
					}
					var json_parsed = JSON.parse( json );
					if ( json_parsed.globalSettings !== undefined && json_parsed.workspace !== undefined ) {
						if ( json_parsed.workspace.structures !== undefined ) {
							if ( json_parsed.globalSettings.displayHeader !== undefined && json_parsed.globalSettings.displayFooter !== undefined ) {
								var nb_structures = 0;
								if ( json_parsed.globalSettings.displayHeader ) {
									nb_structures++;
								}
								if ( json_parsed.globalSettings.displayFooter ) {
									nb_structures++;
								}
								if ( json_parsed.workspace.structures.length > nb_structures ) {
									return json;
								}
							}
						}
					}
				}
				catch ( e ) {

				}
			}
			return '';
		}

		function content_email_types( content_email_json, content_email_html, content_email_txt ) {
			if ( content_email_json !== '' && content_email_txt === '' ) {
				return 'Html (Emailbuilder)';
			}
			else if ( content_email_json !== '' && content_email_txt !== '' ) {
				return 'Html (Emailbuilder) / Text';
			}
			else if ( content_email_json === '' && content_email_html === '' && content_email_txt !== '' ) {
				return 'Text';
			}
			else if ( content_email_json === '' && content_email_html !== '' && content_email_txt !== '' ) {
				return 'Html / Text';
			}
			else if ( content_email_json === '' && content_email_html !== '' && content_email_txt === '' ) {
				return 'Html';
			}
			return '';
		}

		function content_email_nb_links( content_email_json, content_email_html, content_email_txt ) {
			var nb_links = 0;
			if ( content_email_json !== '' ) {
				var json_parsed = JSON.parse( content_email_json );
				if ( json_parsed.links !== undefined ) {
					nb_links += json_parsed.links.length;
				}
				
				nb_links += content_email_json.split( '"link":"BUTTON"' ).length - 1;
				nb_links += content_email_json.split( '"link":"TEXT"' ).length - 1;
			}
			else if ( content_email_html !== '' ) {
				nb_links += content_email_html.split( ' href="' ).length - 1;
				nb_links += content_email_html.split( ' href=\'' ).length - 1;
			}
			else if ( content_email_txt !== '' ) {
				nb_links += content_email_txt.split( '((WEBCOPY_LINK))' ).length - 1;
				nb_links += content_email_txt.split( '((UNSUBSCRIBE_LINK))' ).length - 1;
				var links = content_email_txt.match( /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/ig );
				if ( links ) {
					nb_links += links.length;
				}
			}
			return nb_links;
		}

		function content_email_unsubscribe_link( content_email_json, content_email_html, content_email_txt ) {
			if ( content_email_json.indexOf( '{{tracking.unsubscribe}}' ) !== -1 ) {
				return true;
			}
			if ( content_email_json.indexOf( '{{unsubscribe}}' ) !== -1 ) {
				return true;
			}
			var link_unsubscribe = '((UNSUBSCRIBE_LINK))';
			if ( content_email_json.indexOf( link_unsubscribe ) !== -1 ) {
				return true;
			}
			if ( content_email_html.indexOf( link_unsubscribe ) !== -1 ) {
				return true;
			}
			if ( content_email_txt.indexOf( link_unsubscribe ) !== -1 ) {
				return true;
			}
			return false;
		}

		var service = {

			dynamic_content_type: 'article',

			init_emailbuilder: function( is_scenario, nb_posts ) {

				var deferred = $q.defer();

				if ( $rootScope.settings.emailbuilder_installed ) {

					if ( window.location.href.indexOf( 'woocommerce_automated_newsletter' ) !== -1 ) {

						service.dynamic_content_type = 'product';

						$timeout( function() {

							angular.element( '.jackmail email-builder #content-4 span' ).text( $rootScope.translations.dynamic_product );

							angular.element( '.jackmail email-builder #content-4' ).mouseout( function() {
								service.dynamic_product_display();
							} );

						} );

					}

					var static_content_type = 'article';

					window.angularComponentRef.zone.run( function() {
						
						var settings = {
							'environment': 'jackmail',
							'wordpress': {
								'frontApiWpUrl': $rootScope.settings.ajax_url + '?action=jackmail_get_images&key=' + $rootScope.settings.key + '&nonce=' + $rootScope.settings.urls[ 'jackmail_get_images' ],
								'imageLibraryWpUrl': $rootScope.settings.upload_url
							},
							'frontApiUrl': $rootScope.settings.emailbuilder_api_url,
							'imageLibraryUrl': $rootScope.settings.emailbuilder_image_library_url,
							'lang': $rootScope.settings.language
						};
						window.angularComponentRef.component.initEmailBuilder( settings ).then( function() {
							
							window.angularComponentRef.component.configurationService.useWordpressIntegrationProduct = $rootScope.settings.emailbuilder_display_product_button;
							if ( is_scenario === true ) {
								window.angularComponentRef.component.setAutomatedWorkflow( true );
								service.set_nb_posts_content( nb_posts );
							}
							$timeout( function() {
								if ( $rootScope.settings.emailbuilder_display_product_button ) {
									angular.element( '.jackmail email-builder #content-5 span' ).html( $rootScope.translations.woocommerce );
									
								}
								angular.element( '.jackmail email-builder #content-3, .jackmail email-builder #content-5, .jackmail email-builder #content-7' ).mouseenter( function( event ) {
									if ( event.currentTarget.id ) {
										if ( event.currentTarget.id === 'content-3' ) {
											static_content_type = 'article';
										}
										else if ( event.currentTarget.id === 'content-5' ) {
											static_content_type = 'product';
										}
										else if ( event.currentTarget.id === 'content-7' ) {
											static_content_type = 'page';
										}
									}
								} );
							} );
							deferred.resolve();
						} );
						

						if ( window.angularComponentRef.component.dynamicContentService ) {

							var nb_categories = 0;

							window.angularComponentRef.component.dynamicContentService.loadCategories = function() {

								$timeout( function() {
									var title_popin = $rootScope.translations.import_an_article;
									var title_link = $rootScope.translations.article_link;
									var title_button = $rootScope.translations.import_article;
									if ( static_content_type === 'product' ) {
										title_popin = $rootScope.translations.import_a_product;
										title_link = $rootScope.translations.product_link;
										title_button = $rootScope.translations.import_product;
									}
									else if ( static_content_type === 'page' ) {
										title_popin = $rootScope.translations.import_a_page;
										title_link = $rootScope.translations.page_link;
										title_button = $rootScope.translations.import_page;
									}
									angular.element( '.jackmail email-builder choose-article-modal .email-builder__modal-title' ).html( title_popin );
									angular.element( '.jackmail email-builder choose-article-modal .email-builder__modal-footer button' ).click( function() {
										$timeout( function() {
											angular.element( '.jackmail email-builder choose-article-modal section:nth-child(3) > label' ).html( title_link );
											angular.element( '.jackmail email-builder choose-article-modal .email-builder__modal-footer button.btn.btn-color' ).html( title_button );
										} );
									} );
								} );

								var deferred = $q.defer();
								var action = 'jackmail_get_post_categories';
								if ( static_content_type === 'page' ) {
									nb_categories = 0;
									return JSON.parse( '[]' );
								}
								else if ( static_content_type === 'product' ) {
									action = 'jackmail_get_woocommerce_product_categories';
								}
								$http( {
									method: 'POST',
									url: $rootScope.settings.ajax_url,
									data: 'action=' + action + '&key=' + $rootScope.settings.key + '&nonce=' + $rootScope.settings.urls[ action ],
									headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
									transformResponse: [ function( data ) {
										return data;
									} ]
								} ).then(
									function( response ) {
										deferred.resolve( JSON.parse( response.data ) );
										nb_categories = JSON.parse( response.data ).length;
									},
									function( response ) {
										UrlService.error_message( response );
									}
								);
								return deferred.promise;
							};

							window.angularComponentRef.component.dynamicContentService.loadArticles = function( title, categories ) {
								if ( title === undefined ) {
									title = '';
								}
								if ( categories === undefined || categories === null || categories.length === nb_categories ) {
									categories = [];
								}
								categories = $rootScope.join( categories );
								var deferred = $q.defer();
								var action = 'jackmail_get_posts';
								if ( static_content_type === 'page' ) {
									action = 'jackmail_get_pages';
								}
								else if ( static_content_type === 'product' ) {
									action = 'jackmail_get_woocommerce_products';
								}
								$http( {
									method: 'POST',
									url: $rootScope.settings.ajax_url,
									data: 'action=' + action + '&title=' + title + '&categories=' + categories + '&key=' + $rootScope.settings.key + '&nonce=' + $rootScope.settings.urls[ action ],
									headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
									transformResponse: [ function( data ) {
										return data;
									} ]
								} ).then(
									function( response ) {
										deferred.resolve( JSON.parse( response.data ) );
									},
									function( response ) {
										UrlService.error_message( response );
									}
								);
								return deferred.promise;
							};

							window.angularComponentRef.component.dynamicContentService.loadArticle = function( id ) {
								if ( id !== undefined ) {
									var deferred = $q.defer();
									var action = 'jackmail_get_post_or_page_full_content';
									if ( static_content_type === 'product' ) {
										action = 'jackmail_get_woocommerce_product_full_content';
									}
									$http( {
										method: 'POST',
										url: $rootScope.settings.ajax_url,
										data: 'action=' + action + '&post_id=' + id + '&key=' + $rootScope.settings.key + '&nonce=' + $rootScope.settings.urls[ action ],
										headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
										transformResponse: [ function( data ) {
											return data;
										} ]
									} ).then(
										function( response ) {
											deferred.resolve( JSON.parse( response.data ) );
										},
										function( response ) {
											UrlService.error_message( response );
										}
									);
									return deferred.promise;
								}
							};

							if ( window.angularComponentRef.component.imageUploadManager !== undefined ) {
								var imageStorageService = window.angularComponentRef.component.imageUploadManager;
							}
							else {
								var imageStorageService = window.angularComponentRef.component.imageStorageService;
							}

							imageStorageService.base64ToUrl = function( base64 ) {
								if ( base64 !== undefined ) {
									var deferred = $q.defer();
									var action = 'jackmail_save_image';
									var data = {
										'image': base64
									};
									var params = angular.element.param( data ) + '&action=' + action + '&key=' + $rootScope.settings.key + '&nonce=' + $rootScope.settings.urls[ action ];
									$http( {
										method: 'POST',
										url: $rootScope.settings.ajax_url,
										data: params,
										headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
										transformResponse: [ function( data ) {
											return data;
										} ]
									} ).then(
										function( response ) {
											var json = JSON.parse( response.data );
											if ( json.success ) {
												deferred.resolve( json.url );
											}
											else {
												UrlService.error_message( '' );
											}
										},
										function( response ) {
											UrlService.error_message( response );
										}
									);
									return deferred.promise;
								}

							};
							

						}

					} );
					
					service.dynamic_product_display();
					angular.element( '.jackmail email-builder .menu-title' ).click( function() {
						service.refresh_email_content_height();
					} );
				}
				else {
					deferred.reject();
				}
				return deferred.promise;
			},

			update_list_fields: function( fields ) {
				if ( $rootScope.settings.emailbuilder_installed ) {
					var data_parameters = {
						'fields': $rootScope.join_fields( fields )
					};
					UrlService.post_data( 'jackmail_get_fields_and_ids', data_parameters, function( data ) {
						window.angularComponentRef.zone.run( function() {
							window.angularComponentRef.component.loadCustomTags( data );
						} );
					}, function() {

					} );
				}
			},

			get_nb_posts_content: function() {
				return window.angularComponentRef.zone.run( function() {
					var nb_posts_content = window.angularComponentRef.component.getArticleCount();
					return nb_posts_content.toString();
				} );
			},

			set_nb_posts_content: function( article_count ) {
				window.angularComponentRef.zone.run( function() {
					if ( window.angularComponentRef.component.setArticleCount !== undefined ) {
						window.angularComponentRef.component.setArticleCount( article_count );
					}
				} );
				service.dynamic_product_display();
			},

			activate_link_tracking: function() {
				if ( $rootScope.settings.emailbuilder_installed ) {
					window.angularComponentRef.component.setLinkTracking( true );
				}
			},

			deactivate_link_tracking: function() {
				if ( $rootScope.settings.emailbuilder_installed ) {
					window.angularComponentRef.component.setLinkTracking( false );
				}
			},

			set_email_content_editor: function( current_content_email_type, content ) {
				init_editor_if_needed( current_content_email_type );
				editor.content = content;
				refresh_html_editor();
			},

			set_emailbuilder_json: function( json ) {
				window.angularComponentRef.zone.run( function() {
					if ( json === 'reset' ) {
						window.angularComponentRef.component.resetWorkspace();
					}
					else if ( json ) {
						content_is_loading = json;
						window.angularComponentRef.component.loadContent( json ).then( function() {
							content_is_loading = false;
							
						} );
					}
				} );
				service.dynamic_product_display();
			},

			display: function( current_content_email_type ) {
				if ( current_content_email_type === 'emailbuilder' ) {
					display_emailbuilder();
				}
				else {
					display_email_content_editor( current_content_email_type );
				}
			},

			set: function( content_email_json, content_email_html, content_email_txt ) {
				var current_content_email_type = '';
				if ( $rootScope.settings.emailbuilder_installed && ( content_email_json !== '' || ( content_email_html === '' && content_email_txt === '' ) ) ) {
					current_content_email_type = 'emailbuilder';
					if ( content_email_json !== '' ) {
						service.set_emailbuilder_json( content_email_json );
					}
					
				}
				else {
					if ( content_email_txt !== '' ) {
						current_content_email_type = 'txt';
						service.set_email_content_editor( current_content_email_type, content_email_txt );
					}
					else if ( content_email_html !== '' ) {
						current_content_email_type = 'html';
						service.set_email_content_editor( current_content_email_type, content_email_html );
					}
					else {
						current_content_email_type = 'html';
					}
					display_email_content_editor( current_content_email_type );
				}
				return {
					'content_email_json': content_email_json,
					'content_email_html': content_email_html,
					'content_email_txt': content_email_txt,
					'current_content_email_type': current_content_email_type, 
					'content_email_types': content_email_types( content_email_json, content_email_html, content_email_txt ),
					'content_email_nb_links': content_email_nb_links( content_email_json, content_email_html, content_email_txt ),
					'content_email_unsubscribe_link': content_email_unsubscribe_link( content_email_json, content_email_html, content_email_txt )
				};
			},

			init_and_display_emailbuilder: function( content_email_json ) {
				var content_email = service.set( content_email_json, '', '' );
				display_emailbuilder();
				return content_email;
			},

			change_current_content_email_type: function( old_current_content_email_type, new_current_content_email_type, content_email_json, content_email_html, content_email_txt ) {
				if ( old_current_content_email_type === 'txt' ) {
					content_email_txt = service.get_email_content_editor( new_current_content_email_type );
				}
				if ( old_current_content_email_type === 'html' && new_current_content_email_type === 'txt' ) {
					content_email_html = service.get_email_content_editor( new_current_content_email_type );
				}
				if ( new_current_content_email_type === 'emailbuilder' ) {
					service.set_email_content_editor( new_current_content_email_type, '' );
					content_email_html = '';
					display_emailbuilder();
				}
				else if ( new_current_content_email_type === 'html' ) {
					if ( content_email_json !== '' ) {
						content_email_html = '';
						content_email_json = '';
						service.set_emailbuilder_json( 'reset' );
					}
					service.set_email_content_editor( new_current_content_email_type, content_email_html );
					display_email_content_editor( new_current_content_email_type );
				}
				else if ( new_current_content_email_type === 'txt' ) {
					service.set_email_content_editor( new_current_content_email_type, content_email_txt );
					display_email_content_editor( new_current_content_email_type );
				}
				return {
					'content_email_json': content_email_json,
					'content_email_html': content_email_html,
					'content_email_txt': content_email_txt,
					'current_content_email_type': new_current_content_email_type, 
					'content_email_types': content_email_types( content_email_json, content_email_html, content_email_txt ),
					'content_email_nb_links': content_email_nb_links( content_email_json, content_email_html, content_email_txt ),
					'content_email_unsubscribe_link': content_email_unsubscribe_link( content_email_json, content_email_html, content_email_txt )
				};
			},

			refresh_content_email: function( current_content_email_type, content_email_json, content_email_html, content_email_txt ) {
				var content_email_changes = false;
				var old_content_email_json = angular.copy( content_email_json );
				var old_last_content_email_html = angular.copy( content_email_html );
				var old_last_content_email_txt = angular.copy( content_email_txt );
				if ( current_content_email_type === 'emailbuilder' ) {
					content_email_json = get_emailbuilder_json();
					content_email_html = '';
				}
				else {
					if ( current_content_email_type === 'html' ) {
						content_email_html = service.get_email_content_editor( current_content_email_type );
					}
					else {
						content_email_txt = service.get_email_content_editor( current_content_email_type );
					}
				}
				if ( old_content_email_json !== content_email_json || old_last_content_email_html !== content_email_html || old_last_content_email_txt !== content_email_txt ) {
					content_email_changes = true;
				}
				return {
					'content_email_json': content_email_json,
					'content_email_html': content_email_html,
					'content_email_txt': content_email_txt,
					'content_email_changes': content_email_changes, 
					'content_email_types': content_email_types( content_email_json, content_email_html, content_email_txt ),
					'content_email_nb_links': content_email_nb_links( content_email_json, content_email_html, content_email_txt ),
					'content_email_unsubscribe_link': content_email_unsubscribe_link( content_email_json, content_email_html, content_email_txt )
				};
			},

			hide_emailbuilder: function() {
				$rootScope.show_emailbuilder = false;
				refresh_email_content_height = false;
			},

			get_email_content_editor: function( current_content_email_type ) {
				init_editor_if_needed( current_content_email_type );
				return editor.content;
			},

			insert_email_content_editor_customize: function( current_content_email_type, content ) {
				if ( editor !== undefined ) {
					if ( current_content_email_type === 'html' && content === 'WEBCOPY_LINK' ) {
						editor.addToCursor( '<a href="((' + content + '))">' + $rootScope.translations.webcopy + '</a>' );
					}
					else if ( current_content_email_type === 'html' && content === 'UNSUBSCRIBE_LINK' ) {
						editor.addToCursor( '<a href="((' + content + '))">' + $rootScope.translations.unsubscribe + '</a>' );
					}
					else {
						editor.addToCursor( '((' + content + '))' );
					}
				}
			},

			hide_emailbuilder_menu: function() {
				window.angularComponentRef.zone.run( function() {
					window.angularComponentRef.component.hideSideMenu = true;
				} );
			},

			dynamic_product_display: function() {
				
				if ( service.dynamic_content_type === 'product' ) {
					$timeout( function() {
						angular.element( '.jackmail.ng-scope email-builder .email-builder__super-content .email-builder__top-menu > div.email-builder__top-menu-element.email-builder__top-menu-size' ).each( function() {
							var title = angular.element( this ).html();
							if ( title.toLowerCase().indexOf( 'article' ) !== -1 ) {
								title = title.split( 'WP' );
								title = $rootScope.translations.dynamic_product + ' WP ' + title[ 1 ];
								angular.element( this ).html( title );
							}
						} );
						angular.element( '.jackmail.jackmail_angular email-builder .email-builder__super-content .email-builder__super-content-overlay button' ).each( function() {
							var title = angular.element( this ).html();
							if ( title.toLowerCase().indexOf( 'article' ) !== -1 ) {
								title = title.replace( 'Article', $rootScope.translations.product );
								angular.element( this ).html( title );
							}
						} );
					} );
				}
			},

			refresh_email_content_height: function() {
				if ( refresh_email_content_height || $rootScope.show_emailbuilder ) {
					var window_height = parseInt( angular.element( $window ).height() );
					var scroll = parseInt( angular.element( $window ).scrollTop() );
					if ( $rootScope.show_emailbuilder ) {
						var jackmail_position_top = Math.round( angular.element( '.jackmail.jackmail_angular #jackmail_emailbuilder_container' ).offset().top );
						var footer_height = 0;
						angular.element( '.jackmail.jackmail_angular .jackmail_footer' ).each( function() {
							footer_height = parseInt( angular.element( this ).height() );
						} );
						var height = window_height - footer_height - jackmail_position_top + scroll;
						angular.element( '.jackmail #jackmail_emailbuilder_container' ).css( 'height', height + 'px' );
					}
					else {
						angular.element( '.jackmail #jackmail_content_email' ).each( function() {
							var position = Math.round( angular.element( this ).offset().top );
							var footerHeight = 40;
							angular.element( '.jackmail .jackmail_footer' ).each( function() {
								footerHeight = parseInt( angular.element( '.jackmail .jackmail_footer' ).css( 'height' ) );
							} );
							var height = parseInt( window_height - position + scroll - footerHeight - 25 );
							angular.element( '.jackmail #jackmail_content_email > div' ).css( 'height', height + 'px' );
						} );
					}
				}
			}

		};

		return service;

	} ] );

angular.module( 'jackmail.services' ).factory( 'ExportService', [
	'$filter', '$timeout', '$rootScope',
	function( $filter, $timeout, $rootScope ) {

		var service = {

			escape_double_quote: function( string ) {
				if ( string.indexOf( '"' ) === -1 && string.indexOf( ' ' ) === -1 ) {
					return string;
				}
				return '"' + string.replace( /"/g, '""' ) + '"';
			},

			blacklist_type_string: function( blacklist_id ) {
				if ( blacklist_id !== '0' ) {
					if ( $rootScope.settings.blacklist_type_bounces === blacklist_id ) {
						return 'bounce';
					}
					else if ( $rootScope.settings.blacklist_type_complaints === blacklist_id ) {
						return 'complaint';
					}
					else if ( $rootScope.settings.blacklist_type_unsubscribes === blacklist_id ) {
						return 'unsubscribe';
					}
				}
				return '';
			},

			get_contact_file: function( list, export_type ) {
				var csv = '';
				var has_blacklist = false;
				var i;
				var header = list.list;
				var header_fields = JSON.parse( header.fields );
				var contacts = list.contacts;
				var nb_contacts = contacts.length;
				var nb_header_fields = header_fields.length;
				for ( i = 0; i < nb_contacts; i++ ) {
					if ( export_type === 'all' || ( export_type === 'selection' && contacts[ i ].selected ) ) {
						if ( contacts[ i ].blacklist !== '0' ) {
							has_blacklist = true;
							break;
						}
					}
				}
				csv += service.escape_double_quote( $rootScope.translations.email.toUpperCase() );
				for ( i = 0; i < nb_header_fields; i++ ) {
					csv += ';' + service.escape_double_quote( header_fields[ i ] );
				}
				if ( has_blacklist ) {
					csv += ';BLACKLIST';
				}
				csv += "\n";
				for ( i = 0; i < nb_contacts; i++ ) {
					if ( export_type === 'all' || ( export_type === 'selection' && contacts[ i ].selected ) ) {
						csv += service.escape_double_quote( contacts[ i ].email );
						var j;
						for ( j = 0; j < nb_header_fields; j++ ) {
							csv += ';';
							if ( contacts[ i ][ 'field' + ( j + 1 ) ] !== undefined ) {
								csv += service.escape_double_quote( contacts[ i ][ 'field' + ( j + 1 ) ] );
							}
						}
						if ( has_blacklist ) {
							csv += ';' + service.blacklist_type_string( contacts[ i ].blacklist );
						}
						csv += "\n";
					}
				}
				return csv;
			},

			export_contact_file_multiple_data: function( data ) {
				var i;
				var nb_parts = data.length;
				var new_data = {
					'list': {},
					'contacts': [],
					'nb_contacts': '',
					'nb_contacts_search': ''
				};
				var j;
				var nb_contacts;
				for ( i = 0; i < nb_parts; i++ ) {
					nb_contacts = data[ i ].contacts.length;
					for ( j = 0; j < nb_contacts; j++ ) {
						new_data.contacts.push( data[ i ].contacts[ j ] );
					}
					if ( i === nb_parts - 1 ) {
						new_data.list = data[ i ].list;
						new_data.nb_contacts = data[ i ].nb_contacts;
						new_data.nb_contacts_search = data[ i ].nb_contacts_search;
					}
				}
				service.export_contact_file( new_data, 'all' );
			},

			export_contact_file: function( list, export_type ) {
				var csv = '\ufeff' + service.get_contact_file( list, export_type );
				var current_date = new Date();
				var filename = 'export-' + $filter( 'formatedDateFromTimestampToTimezone' )( current_date, 'file_name' ) + '.csv';
				var blob = new Blob( [ csv ], { type: 'text/csv;charset=utf-8;' } );
				if ( navigator.msSaveBlob ) {
					navigator.msSaveBlob( blob, filename );
				}
				else {
					var data = URL.createObjectURL( blob );
					angular.element( '.jackmail_download' ).attr( { 'href': data, 'download': filename } );
					$timeout( function() {
						angular.element( '.jackmail_download' )[ 0 ].click();
						angular.element( '.jackmail_download' ).attr( { 'href': '', 'download': '' } );
					} );
				}
			}

		};

		return service;

	} ] );

angular.module( 'jackmail.services' ).factory( 'GridService', [
	'$window', '$rootScope',
	function( $window, $rootScope ) {

		var GridService = function() {

			var service = {

				nb_selected: 0,

				grid_class: '',

				display_columns_button: false,

				grid_range_by_order: false,

				grid_range_by_item: '',

				load_interval: 100,

				nb_lines_grid: 100,

				begin: 0,

				grid_classes: [],

				init_columns_list: function( fields ) {
					var nb_columns = fields.length;
					var var_grid_classes = [];
					var var_grid_class = '';
					var i;
					var_grid_classes[ 0 ] = true;
					var firstname_or_lastname = false;
					for ( i = 0; i < nb_columns; i++ ) {
						if ( fields[ i ] === ( $rootScope.translations.name ).toUpperCase() || fields[ i ] === ( $rootScope.translations.firstname ).toUpperCase() || fields[ i ] === ( $rootScope.translations.lastname ).toUpperCase() ) {
							var_grid_classes[ i + 1 ] = true;
							firstname_or_lastname = true;
						}
						else {
							var_grid_classes[ i + 1 ] = false;
							var_grid_class += ' jackmail_hide_column_' + ( i + 1 );
						}
					}
					if ( !firstname_or_lastname ) {
						var_grid_class = '';
						for ( i = 0; i < nb_columns; i++ ) {
							if ( i < 2 ) {
								var_grid_classes[ i + 1 ] = true;
							}
							else {
								var_grid_classes[ i + 1 ] = false;
								var_grid_class += ' jackmail_hide_column_' + ( i + 1 );
							}
						}
					}
					service.grid_classes = var_grid_classes;
					service.grid_class = var_grid_class;
				},

				init_columns: function( nb_columns ) {
					var var_grid_classes = [];
					var var_grid_class = '';
					var i;
					for ( i = 0; i < nb_columns; i++ ) {
						if ( i < 5 ) {
							var_grid_classes[ i ] = true;
						}
						else {
							var_grid_classes[ i ] = false;
							var_grid_class += ' jackmail_hide_column_' + i;
						}
					}
					service.grid_classes = var_grid_classes;
					service.grid_class = var_grid_class;
				},

				set_nb_selected: function( value ) {
					service.nb_selected = value;
				},

				set_nb_lines_grid: function( nb_lines_grid ) {
					service.nb_lines_grid = nb_lines_grid;
				},

				reset_nb_lines_grid: function() {
					service.nb_lines_grid = 100;
				},

				increase_begin: function() {
					service.begin += 100;
				},

				reset_begin: function() {
					service.begin = 0;
				},

				merge_arrays: function( array1, array2 ) {
					var i;
					var nb = array2.length;
					for ( i = 0; i < nb; i++ ) {
						array1.push( array2[ i ] );
					}
					return array1;
				},

				refresh_grid_max_height: function( type ) {
					angular.element( '.jackmail_grid_content' ).each( function() {
						var position = Math.round( angular.element( this ).offset().top );
						var footerHeight = 0;
						angular.element( '.jackmail_footer' ).each( function() {
							footerHeight = parseInt( angular.element( '.jackmail_footer' ).css( 'height' ) );
						} );
						var height = parseInt( angular.element( $window ).height() );
						var scroll = parseInt( angular.element( $window ).scrollTop() );
						var grid_height = parseInt( height - position + scroll - footerHeight - 65 );
						angular.element( this ).css( 'maxHeight', grid_height + 'px' );
						if ( type === 'resize' || type === 'refresh' ) {
							var width = parseInt( angular.element( this ).css( 'width' ) );
							if ( width !== 0 ) {
								angular.element( this ).find( ' > table' ).css( 'width', width );
							}
						}
					} );
					if ( type === 'resize' || type === 'refresh' ) {
						angular.element( '.jackmail_grid_content_defined' ).each( function() {
							var width = parseInt( angular.element( this ).css( 'width' ) );
							if ( width !== 0 ) {
								angular.element( this ).find( ' > table' ).css( 'width', width );
							}
						} );
					}
				},

				display_or_hide_column: function( key ) {
					service.grid_classes[ key ] = !service.grid_classes[ key ];
					service.grid_column_refresh();
				},

				display_column: function( key ) {
					service.grid_classes[ key ] = true;
					service.grid_column_refresh();
				},

				grid_column_refresh: function() {
					var i;
					var nb_columns = service.grid_classes.length;
					var grid_class = '';
					for ( i = 0; i < nb_columns; i++ ) {
						if ( !service.grid_classes[ i ] ) {
							grid_class += ' jackmail_hide_column_' + i;
						}
					}
					service.grid_class = grid_class;
				},

				grid_range: function( items, i ) {
					service.grid_range_by_order = !service.grid_range_by_order;
					service.grid_range_by_item = i;
					var filtered = service.grid_order_by( items, i, 'ASC' );
					if ( service.grid_range_by_order ) {
						filtered.reverse();
					}
					return filtered;
				},

				grid_range_from_server: function( i ) {
					service.grid_range_by_order = !service.grid_range_by_order;
					service.grid_range_by_item = i;
				},

				grid_order_by: function( items, i, order ) {
					var filtered = [];
					angular.forEach( items, function( item ) {
						filtered.push( item );
					} );
					filtered.sort( function( a, b ) {
						return ( a[ i ] > b[ i ] ? 1 : -1 );
					} );
					if ( order === 'DESC' ) {
						filtered.reverse();
					}
					return filtered;
				},

				display_or_hide_columns_button: function() {
					service.display_columns_button = !service.display_columns_button;
				},

				hide_columns_button: function() {
					service.display_columns_button = false;
				},

				grid_select_or_unselect_row: function( rows, key ) {
					if ( rows[ key ].selected ) {
						rows[ key ].selected = false;
						service.nb_selected--;
					}
					else {
						service.nb_selected++;
						rows[ key ].selected = true;
					}
					return rows;
				},

				grid_select_or_unselect_all: function( rows ) {
					if ( service.nb_selected === rows.length ) {
						var select_all = false;
						service.nb_selected = 0;
					}
					else {
						var select_all = true;
						service.nb_selected = rows.length;
					}
					var i;
					var nb_rows = rows.length;
					for ( i = 0; i < nb_rows; i++ ) {
						rows[ i ].selected = select_all;
					}
					return rows;
				},

				grid_select_or_unselect_all_with_field_restriction: function( rows, field ) {
					var i;
					var nb_lists = rows.length;
					var nb_selected = 0;
					for ( i = 0; i < nb_lists; i++ ) {
						if ( !rows[ i ][ field ] ) {
							nb_selected++;
						}
					}
					if ( service.nb_selected === nb_selected ) {
						var select_all = false;
						service.set_nb_selected( 0 );
					}
					else {
						var select_all = true;
						service.set_nb_selected( nb_selected );
					}
					for ( i = 0; i < nb_lists; i++ ) {
						if ( !rows[ i ][ field ] ) {
							rows[ i ].selected = select_all;
						}
					}
					return rows;
				},

				grid_select_all: function( rows ) {
					service.nb_selected = rows.length;
					var i;
					var nb_rows = rows.length;
					for ( i = 0; i < nb_rows; i++ ) {
						rows[ i ].selected = true;
					}
					return rows;
				},

				grid_select_all_new: function( rows ) {
					var nb_selected = 0;
					var i;
					var nb_rows = rows.length;
					for ( i = 0; i < nb_rows; i++ ) {
						if ( rows[ i ].selected === undefined ) {
							rows[ i ].selected = true;
							nb_selected++;
						}
					}
					service.nb_selected = nb_selected;
					return rows;
				},

				grid_select_only_row: function( rows, key ) {
					service.nb_selected = 1;
					rows[ key ].selected = true;
					var i;
					var nb_rows = rows.length;
					for ( i = 0; i < nb_rows; i++ ) {
						if ( i !== key ) {
							rows[ i ].selected = false;
						}
					}
					return rows;
				},

				grid_select_only_campaign_ids: function( rows, ids ) {
					var i;
					var nb_selected_temp = 0;
					var i;
					var nb_rows = rows.length;
					for ( i = 0; i < nb_rows; i++ ) {
						var j;
						if ( ids.indexOf( rows[ i ].id + '' + rows[ i ].type ) !== -1 ) {
							rows[ i ].selected = true;
							nb_selected_temp++;
						}
						else {
							rows[ i ].selected = false;
						}
					}
					service.nb_selected = nb_selected_temp;
					return rows;
				}

			};

			service.init_columns( 10 );

			return service;

		};

		return GridService;

	} ] );

angular.module( 'jackmail.services' ).factory( 'PluginService', [
	'$rootScope',
	function( $rootScope ) {

		var service = {

			get_compatible_plugins: function() {
				return {
					'bloom': {
						'active': false,
						'selected': false,
						'name': 'Bloom',
						'description': $rootScope.translations.plugin_bloom_description
					},
					'contactform7': {
						'active': false,
						'selected': false,
						'name': 'Contact Form 7',
						'description': $rootScope.translations.plugin_contactform7_description
					},
					'formidableforms': {
						'active': false,
						'selected': false,
						'name': 'Formidable Forms',
						'description': $rootScope.translations.plugin_formidableforms_description
					},
					'gravityforms': {
						'active': false,
						'selected': false,
						'name': 'Gravity Forms',
						'description': $rootScope.translations.plugin_gravityforms_description
					},
					'mailpoet2': {
						'active': false,
						'selected': false,
						'name': 'MailPoet 2',
						'description': $rootScope.translations.plugin_mailpoet2_description
					},
					'mailpoet3': {
						'active': false,
						'selected': false,
						'name': 'MailPoet 3',
						'description': $rootScope.translations.plugin_mailpoet3_description
					},
					'ninjaforms': {
						'active': false,
						'selected': false,
						'name': 'Ninja Forms',
						'description': $rootScope.translations.plugin_ninjaforms_description
					},
					'popupbysupsystic': {
						'active': false,
						'selected': false,
						'name': 'PopUp by Supsystic',
						'description': $rootScope.translations.plugin_popupbysupsystic_description
					},
					'woocommerce': {
						'active': false,
						'selected': false,
						'name': 'WooCommerce',
						'description': $rootScope.translations.plugin_woocommerce_description
					}
				};
			},

			get_plugins_to_display: function( plugins, data ) {
				var i;
				var nb = data.length;
				for ( i = 0; i < nb; i++ ) {
					plugins[ data[ i ].plugin ].active = true;
					if ( data[ i ].selected ) {
						plugins[ data[ i ].plugin ].selected = true;
					}
				}
				return plugins;
			},

			get_active_plugins: function( plugins ) {
				var active_plugins = [];
				if ( plugins.bloom.active && plugins.bloom.selected ) {
					active_plugins.push( 'bloom' );
				}
				if ( plugins.contactform7.active && plugins.contactform7.selected ) {
					active_plugins.push( 'contactform7' );
				}
				if ( plugins.formidableforms.active && plugins.formidableforms.selected ) {
					active_plugins.push( 'formidableforms' );
				}
				if ( plugins.gravityforms.active && plugins.gravityforms.selected ) {
					active_plugins.push( 'gravityforms' );
				}
				if ( plugins.mailpoet2.active && plugins.mailpoet2.selected ) {
					active_plugins.push( 'mailpoet2' );
				}
				if ( plugins.mailpoet3.active && plugins.mailpoet3.selected ) {
					active_plugins.push( 'mailpoet3' );
				}
				if ( plugins.ninjaforms.active && plugins.ninjaforms.selected ) {
					active_plugins.push( 'ninjaforms' );
				}
				if ( plugins.popupbysupsystic.active && plugins.popupbysupsystic.selected ) {
					active_plugins.push( 'popupbysupsystic' );
				}
				if ( plugins.woocommerce.active && plugins.woocommerce.selected ) {
					active_plugins.push( 'woocommerce' );
				}
				return active_plugins;
			}

		};

		return service;

	} ] );

angular.module( 'jackmail.services' ).factory( 'UrlService', [
	'$http', '$rootScope', '$location', '$timeout', '$window', '$q',
	function( $http, $rootScope, $location, $timeout, $window, $q ) {

		var service = {

			post_data: function( action, data, callback_success, callback_error ) {
				service.request_data();
				var params = angular.element.param( data ) + '&action=' + action + '&key=' + $rootScope.settings.key + '&nonce=' + $rootScope.settings.urls[ action ];
				$http( {
					method: 'POST',
					url: $rootScope.settings.ajax_url,
					data: params,
					headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
					transformResponse: [ function( data ) {
						return data;
					} ]
				} ).then(
					function( response ) {
						service.check_received_data( response.data, callback_success, callback_error );
					},
					function( response ) {
						console.log( response );
						service.response_data();
						service.error_message( response );
						callback_error( response.data );
					}
				);
			},

			get_file_data: function( url, callback_success, callback_error ) {
				service.request_data();
				$http( {
					method: 'GET',
					url: url,
					headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
					transformResponse: [ function( data ) {
						return data;
					} ]
				} ).then(
					function( response ) {
						service.response_data();
						callback_success( response.data );
					},
					function( response ) {
						console.log( response );
						service.response_data();
						service.error_message( response );
						callback_error( response.data );
					}
				);
			},

			post_multiple_data: function( action, data, message, callback_success, callback_error ) {
				var current_part = 0;
				var nb_parts = data.length;

				if ( nb_parts > 0 ) {
					service.request_data();
					var result = [];
					next_part();
				}

				function next_part() {
					if ( message !== '' ) {
						$rootScope.jackmail_success = true;
						$rootScope.jackmail_error = false;
						var current_message = message;
						if ( nb_parts > 1 ) {
							current_message += ' (' + parseInt( ( current_part + 1 ) * ( 100 / nb_parts ) ) + ' %)';
						}
						$rootScope.jackmail_message = current_message;
					}
					var params = angular.element.param( data[ current_part ] ) + '&action=' + action + '&key=' + $rootScope.settings.key + '&nonce=' + $rootScope.settings.urls[ action ];
					$http( {
						method: 'POST',
						url: $rootScope.settings.ajax_url,
						data: params,
						headers: { 'Content-Type': 'application/x-www-form-urlencoded' },
						transformResponse: [ function( data ) {
							return data;
						} ]
					} ).then(
						function( response ) {
							current_part++;
							if ( response.data !== '{"success":true}' && response.data !== '{"success":false}' ) {
								result.push( JSON.parse( response.data ) );
							}
							if ( current_part < nb_parts ) {
								next_part();
							}
							else {
								service.response_data();
								if ( result.length === 0 ) {
									callback_success( { 'success': true } );
								}
								else {
									callback_success( result );
								}
								if ( message !== '' ) {
									$rootScope.jackmail_success = false;
									$rootScope.jackmail_message = '';
								}
							}
						},
						function( response ) {
							console.log( response );
							service.error_message( response );
							callback_error( { 'success': false } );
						}
					);
				}

			},

			check_received_data: function( data, callback_success, callback_error ) {
				service.response_data();
				try {
					data = JSON.parse( data );
					callback_success( data );
				}
				catch ( e ) {
					console.log( e );
					service.error_message( '' );
					callback_error( { 'success': false } );
				}
			},

			error_message: function( response ) {
				if ( response.status !== undefined && response.statusText !== undefined ) {
					if ( response.status >= 300 ) {
						var message = $rootScope.translations.an_error_occurred + ' (' + $rootScope.translations.server_message;
						message += ' ' + response.status;
						if ( response.statusText !== '' ) {
							message += ' - ' + response.statusText;
						}
						message += ')';
						$rootScope.display_success_error( false, '', message );
					}
				}
			},

			request_data: function() {
				$timeout( function() {
					$rootScope.nb_loading++;
					if ( $rootScope.nb_loading === 1 ) {
						$rootScope.loading = true;
					}
				} );
			},

			response_data: function() {
				$timeout( function() {
					$rootScope.nb_loading--;
					if ( $rootScope.nb_loading === 0 ) {
						$rootScope.loading = false;
					}
				} );
			},

			change_url_parameters_without_reload: function( url ) {
				if ( $rootScope.settings.analytics ) {
					$window.ga( 'send', 'pageview', $location.path() );
				}
				$location.path( url, false );
			},

			change_url_parameters_without_reload_and_history: function( url ) {
				$location.path( url, false ).replace();
			}

		};

		return service;

	} ] );

angular.module( 'jackmail.services' ).factory( 'VerificationService', [
	function() {
		return {

			email: function( email ) {
				var regex = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
				if ( regex.test( email ) ) {
					return true;
				}
				return false;
			},

			differents_arrays: function( a, b ) {
				delete a[ '$$hashKey' ];
				delete b[ '$$hashKey' ];
				if ( JSON.stringify( a ) === JSON.stringify( b ) ) {
					return false;
				}
				return true;
			}

		};
	} ] );