<?php


class Jackmail_List_And_Campaign extends Jackmail_List_And_Campaign_Common_Core {

	public function __construct( Jackmail_Core $core ) {

		$this->core = $core;

		if ( $this->core->is_admin() ) {

			add_action( 'wp_ajax_jackmail_export_contacts_selection', array( $this, 'export_contacts_selection_callback' ) );

			add_action( 'wp_ajax_jackmail_import_contacts', array( $this, 'import_contacts_callback' ) );

			add_action( 'wp_ajax_jackmail_update_contact_email', array( $this, 'update_contact_email_callback' ) );

			add_action( 'wp_ajax_jackmail_update_contact_field', array( $this, 'update_contact_field_callback' ) );

			add_action( 'wp_ajax_jackmail_unsubscribe_contact', array( $this, 'unsubscribe_contact_callback' ) );

			add_action( 'wp_ajax_jackmail_unblacklist_contact', array( $this, 'unblacklist_contact_callback' ) );

			add_action( 'wp_ajax_jackmail_add_header_column', array( $this, 'add_header_column_callback' ) );

			add_action( 'wp_ajax_jackmail_edit_header_column', array( $this, 'edit_header_column_callback' ) );

			add_action( 'wp_ajax_jackmail_delete_header_column', array( $this, 'delete_header_column_callback' ) );

			add_action( 'wp_ajax_jackmail_delete_contacts_selection', array( $this, 'delete_contacts_selection_callback' ) );

			add_action( 'wp_ajax_jackmail_delete_all_contacts', array( $this, 'delete_all_contacts_callback' ) );

			add_action( 'wp_ajax_jackmail_get_email_lists_detail', array( $this, 'get_email_lists_detail_callback' ) );

			add_action( 'wp_ajax_jackmail_get_email_detail', array( $this, 'get_email_detail_callback' ) );

		}

	}

	public function export_contacts_selection_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( ( isset( $_POST['id_campaign'] ) || isset( $_POST['id_list'] ) ) && isset( $_POST['begin'] ) && isset( $_POST['search'] ) && isset( $_POST['targeting_rules'] ) && isset( $_POST['contacts_selection'] ) ) {
			$begin              = $this->core->request_text_data( $_POST['begin'] );
			$search             = $this->core->request_text_data( $_POST['search'] );
			$targeting_rules    = $this->core->request_text_data( $_POST['targeting_rules'] );
			$contacts_selection = $this->core->request_text_data( $_POST['contacts_selection'] );
			$list               = null;
			$contacts           = array();
			if ( isset( $_POST['id_list'] ) ) {
				$id                  = $this->core->request_text_data( $_POST['id_list'] );
				$table_lists         = "{$wpdb->prefix}jackmail_lists";
				$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id}";
			} else {
				$id                  = $this->core->request_text_data( $_POST['id_campaign'] );
				$table_lists         = "{$wpdb->prefix}jackmail_campaigns";
				$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id}";
			}
			$sql          = "
			SELECT `fields`
			FROM `{$table_lists}`
			WHERE `id` = %s";
			$list         = $wpdb->get_row( $wpdb->prepare( $sql, $id ) );
			$contacts_sql = $this->get_list_or_get_campaign_contacts_sql( $table_list_contacts, 'ALL_FILTERED', '', '', '', $search, $targeting_rules, $contacts_selection, 'NOT_SELECTED' );
			if ( isset( $contacts_sql['sql_contacts'] ) && isset( $contacts_sql['sql_values'] ) ) {
				$sql_contacts   = $contacts_sql['sql_contacts'];
				$sql_values     = $contacts_sql['sql_values'];
				$sql_values[]   = (int) $begin;
				$continue_limit = $this->core->export_send_limit();
				$contacts       = $wpdb->get_results( $wpdb->prepare( $sql_contacts . ' LIMIT %d, ' . $continue_limit, $sql_values ) );
			}
			$data = array(
				'list'     => $list,
				'contacts' => $contacts
			);
			wp_send_json( $data );
		}
		die;
	}

	public function import_contacts_callback() {
		$this->core->check_auth();
		if ( ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) && isset( $_POST['field_separator'] ) && isset( $_POST['email_position'] ) && isset( $_POST['contacts'] ) ) {
			if ( isset( $_POST['id_list'] ) ) {
				$id   = $this->core->request_text_data( $_POST['id_list'] );
				$type = 'list';
			} else {
				$id   = $this->core->request_text_data( $_POST['id_campaign'] );
				$type = 'campaign';
			}
			$field_separator = $this->core->request_text_data( $_POST['field_separator'] );
			if ( $field_separator === 'TAB' ) {
				$field_separator = '    ';
			}
			$email_position = $this->core->request_text_data( $_POST['email_position'] );
			$contacts       = $this->core->request_textarea_data( $_POST['contacts'] );
			$contacts       = str_getcsv( $contacts, "\n" );
			$blacklist_type = 'normal';
			$this->import_contacts( $id, $type, $field_separator, $email_position, $contacts, $blacklist_type );
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function update_contact_email_callback() {
		$this->core->check_auth();
		global $wpdb;
		$json_error = array(
			'success' => false,
			'message' => ''
		);
		if ( ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) && isset( $_POST['email'] ) && isset( $_POST['new_email'] ) ) {
			$email         = $this->core->str_to_lower( $this->core->request_email_data( $_POST['email'] ) );
			$new_email     = $this->core->str_to_lower( $this->core->request_text_data( $_POST['new_email'] ) );
			$data          = array();
			$data['email'] = $new_email;
			$do_action     = array(
				'email'     => $email,
				'new_email' => $new_email
			);
			if ( isset( $_POST['id_list'] ) ) {
				$id                  = $this->core->request_text_data( $_POST['id_list'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id}";
				$type                = 'list';
			} else {
				$id                  = $this->core->request_text_data( $_POST['id_campaign'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id}";
				$data['id_list']     = '0';
				$data['blacklist']   = '0';
				$this->check_if_campaign_is_draft( $id );
				$type = 'campaign';
			}
			if ( $json_error ['message'] === '' ) {
				if ( is_email( $new_email ) ) {
					if ( $email !== $new_email ) {
						$sql         = "
						SELECT COUNT( * )
						FROM `{$table_list_contacts}`
						WHERE `email` = %s";
						$nb_contacts = $wpdb->get_var( $wpdb->prepare( $sql, $new_email ) );
						if ( $nb_contacts === '0' ) {
							if ( $email === '' ) {
								$insert_return = $this->core->insert_list_contact_or_campaign_list_contact( $table_list_contacts, $data );
								if ( $insert_return !== false ) {
									if ( $type === 'list' ) {
										do_action( 'jackmail_list_contact_email_added', $do_action );
									}
								} else {
									wp_send_json_error();
								}
							} else {
								$update_return = $this->core->update_list_contact_or_campaign_list_contact( $table_list_contacts, $data, array(
									'email' => $email
								) );
								if ( $update_return !== false && $update_return > 0 ) {
									if ( $type === 'list' ) {
										do_action( 'jackmail_list_contact_email_updated', $do_action );
									}
								} else {
									wp_send_json_error();
								}
							}
							if ( $type === 'list' ) {
								$this->core->updated_list_contact( $id );
							} else {
								$this->core->updated_campaign_list_contact( $id );
							}
							wp_send_json_success();
						} else {
							$json_error ['message'] = __( 'This email already exists in the list', 'jackmail-newsletters' );
						}
					}
				} else {
					$json_error ['message'] = __( 'A valid email address is required', 'jackmail-newsletters' );
				}
			}
		}
		wp_send_json( $json_error );
		die;
	}

	public function update_contact_field_callback() {
		$this->core->check_auth();
		global $wpdb;
		$json_error = array(
			'success' => false,
			'message' => ''
		);
		if ( ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) && isset( $_POST['email'] ) && isset( $_POST['field_id'] ) && isset( $_POST['field'] ) ) {
			$email = $this->core->str_to_lower( $this->core->request_email_data( $_POST['email'] ) );
			if ( is_email( $email ) ) {
				$field_id = $this->core->request_text_data( $_POST['field_id'] );
				$field    = $this->core->request_text_data( $_POST['field'] );
				if ( isset( $_POST['id_list'] ) ) {
					$id_list             = $this->core->request_text_data( $_POST['id_list'] );
					$table_lists         = "{$wpdb->prefix}jackmail_lists";
					$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}";
					$where_id            = $id_list;
					$type                = 'list';
				} else {
					$id_campaign         = $this->core->request_text_data( $_POST['id_campaign'] );
					$table_lists         = "{$wpdb->prefix}jackmail_campaigns";
					$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
					$where_id            = $id_campaign;
					$this->check_if_campaign_is_draft( $id_campaign );
					$type = 'campaign';
				}
				if ( $json_error ['message'] === '' ) {
					$columns = $this->core->get_table_columns( $table_list_contacts, false );
					if ( in_array( 'field' . $field_id, $columns ) && $email !== '' ) {
						$this->core->update_list_contact_or_campaign_list_contact( $table_list_contacts, array(
							'email'             => $email,
							'field' . $field_id => $field
						), array(
							'email' => $email
						) );
						$do_action = array(
							'email'             => $email,
							'field' . $field_id => $field
						);
						if ( $type === 'list' ) {
							do_action( 'jackmail_list_contact_field_updated', $do_action );
						}
						$this->updated_list_contact_or_campaign_list_contact_time( $table_lists, $where_id );
						wp_send_json_success();
					}
				}
			} else {
				$json_error ['message'] = __( 'A valid email address is required', 'jackmail-newsletters' );
			}
		}
		$json_error ['success'] = false;
		wp_send_json( $json_error );
		die;
	}

	public function unsubscribe_contact_callback() {
		$this->core->check_auth();
		if ( $this->unsubscribe_or_unblacklist_contact( 'unsubscribe' ) ) {
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	public function unblacklist_contact_callback() {
		$this->core->check_auth();
		if ( $this->unsubscribe_or_unblacklist_contact( 'unblacklist' ) ) {
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	private function unsubscribe_or_unblacklist_contact( $action ) {
		global $wpdb;
		if ( ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) && isset( $_POST['email'] ) ) {
			$email       = $this->core->request_email_data( $_POST['email'] );
			$plugin_name = '';
			if ( isset( $_POST['id_list'] ) ) {
				$id_list             = $this->core->request_text_data( $_POST['id_list'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}";
				$sql                 = "SELECT `type` FROM `{$wpdb->prefix}jackmail_lists` WHERE `id` = %s";
				$list                = $wpdb->get_row( $wpdb->prepare( $sql, $id_list ) );
				if ( isset( $list->type ) ) {
					$list_type = $list->type;
					if ( $list_type !== '' ) {
						$list_type = $this->core->explode_data( $list_type );
						if ( isset( $list_type['name'] ) ) {
							$plugin_name = $list_type['name'];
						}
					}
				}
			} else {
				$id_campaign         = $this->core->request_text_data( $_POST['id_campaign'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
			}
			$blacklist = '0';
			if ( $action === 'unsubscribe' ) {
				$blacklist = '1';
			}
			if ( $this->core->is_defined_nb_contacts( $plugin_name ) ) {
				$return = $this->core->update_list_contact_or_campaign_list_contact( $table_list_contacts, array(
					'blacklist' => $blacklist
				), array(
					'email' => $email
				) );
			} else {
				if ( $blacklist === '0' ) {
					$return = $this->core->delete_list_contact_or_campaign_list_contact( $table_list_contacts, array(
						'email' => $email
					) );
				} else {
					$sql_values   = array();
					$sql_values[] = $email;
					$sql_values[] = $blacklist;
					$sql          = "
					INSERT INTO `{$table_list_contacts}` (`email`, `blacklist`)
					VALUES (%s, %s)
					ON DUPLICATE KEY UPDATE `email` = VALUES(`email`), `blacklist` = VALUES(`blacklist`)";
					$return       = $wpdb->query( $wpdb->prepare( $sql, $sql_values ) );
				}
			}
			if ( $return !== false ) {
				return true;
			}
		}
		return false;
	}

	public function get_email_lists_detail_callback() {
		$this->core->check_auth();
		global $wpdb;
		$json = array();
		if ( isset( $_POST['email'] ) ) {
			$email = $this->core->request_email_data( $_POST['email'] );
			if ( isset( $_POST['id_campaign'] ) ) {
				$id_campaign     = $this->core->request_text_data( $_POST['id_campaign'] );
				$table_campaigns = "{$wpdb->prefix}jackmail_campaigns";
				$sql             = "
				SELECT `id`, `name`, `fields`
				FROM `{$table_campaigns}`
				WHERE `id` = %s";
				$campaign        = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
				if ( isset( $campaign->id ) ) {
					$name                = $campaign->name;
					$fields              = $campaign->fields;
					$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
					$contact             = $this->get_list_or_get_campaign_contacts( $table_list_contacts, '0', '1', 'email', 'ASC', $email, '' );
					if ( isset( $contact['nb_contacts_search'] ) && isset( $contact['contacts'][0] ) ) {
						$json[] = array(
							'id'      => $id_campaign,
							'type'    => 'campaign',
							'name'    => __( 'View from', 'jackmail-newsletters' ) . ' "' . $name . '"',
							'contact' => $contact['contacts'][0],
							'fields'  => $fields
						);
					}
				}
			}
			$table_lists = "{$wpdb->prefix}jackmail_lists";
			$sql         = "
			SELECT `id`, `name`, `fields`, `type`
			FROM `{$table_lists}`
			ORDER BY `id` DESC";
			$lists       = $wpdb->get_results( $sql );
			foreach ( $lists as $list ) {
				$contact = array();
				$id_list = $list->id;
				$name    = $list->name;
				$fields  = $list->fields;
				$type    = $list->type;
				if ( $type !== '' ) {
					$plugin = $this->core->explode_data( $type );
					if ( isset( $plugin['name'] ) && isset( $plugin['id'] ) ) {
						$plugin_name = $plugin['name'];
						$plugin_id   = $plugin['id'];
						$contact     = $this->get_plugin_list2( $plugin_name, '0', '1', $plugin_id, $list, 'email', 'ASC', $email );
					}
				} else {
					$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}";
					$contact             = $this->get_list_or_get_campaign_contacts( $table_list_contacts, '0', '1', 'email', 'ASC', $email, '' );
				}
				if ( isset( $contact['nb_contacts_search'] ) && isset( $contact['contacts'][0] ) ) {
					$json[] = array(
						'id'      => $id_list,
						'type'    => 'list',
						'name'    => __( 'View from', 'jackmail-newsletters' ) . ' "' . $name . '"',
						'contact' => $contact['contacts'][0],
						'fields'  => $fields
					);
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function get_email_detail_callback() {
		$this->core->check_auth();
		global $wpdb;
		$json = array(
			'sends'    => 0,
			'clicks'   => 0,
			'opens'    => 0,
			'timeline' => array()
		);
		if ( isset( $_POST['id'] ) && isset( $_POST['email'] ) ) {
			$id    = $this->core->request_text_data( $_POST['id'] );
			$email = $this->core->request_email_data( $_POST['email'] );
			$sql   = "
			SELECT `id`, `id_campaign`
			FROM `{$wpdb->prefix}jackmail_lists`
			WHERE `id` = %s";
			$lists = $wpdb->get_results( $wpdb->prepare( $sql, $id ) );
			if ( count( $lists ) > 0 ) {
				$id_lists = array();
				foreach ( $lists as $list ) {
					$id_lists[] = $this->core->get_campaign_id_list( $list->id, $list->id_campaign );
				}
				$url      = $this->core->get_jackmail_url_analytics() . 'recipient';
				$headers  = array(
					'content-type' => 'application/json',
					'x-auth-token' => $this->core->get_account_token(),
					'accountId'    => $this->core->get_account_id(),
					'userId'       => $this->core->get_user_id()
				);
				$body     = array(
					'beginInterval'  => $this->core->get_iso_date( date( 'Y-m-d H:i:s', strtotime( '-1 day', strtotime( get_option( 'jackmail_install_date' ) ) ) ) ),
					'endInterval'    => $this->core->get_iso_date( date( 'Y-m-d H:i:s', strtotime( '+1 day' ) ) ),
					'listIds'        => $id_lists,
					'recipientEmail' => $email
				);
				$timeout  = 30;
				$response = $this->core->remote_post_retry( $url, $headers, $body, $timeout );
				if ( is_array( $response ) ) {
					if ( isset( $response['body'] ) ) {
						$results = json_decode( $response['body'], true );
						if ( is_array( $results ) ) {
							foreach ( $results as $result ) {
								if ( $this->core->is_statistics_v2() ) {
									if ( isset( $result['nbOpen'] ) && isset( $result['nbHit'] ) && isset( $result['nbSuccess'] ) ) {
										$json             = array(
											'sends'  => $result['nbSuccess'],
											'clicks' => $result['nbHit'],
											'opens'  => $result['nbOpen']
										);
										$json['timeline'] = array();
										$url              = $this->core->get_jackmail_url_analytics() . 'recipient/timeline';
										$timeout          = 30;
										$response         = $this->core->remote_post_retry( $url, $headers, $body, $timeout );
										if ( is_array( $response ) ) {
											if ( isset( $response['body'] ) ) {
												$subresults = json_decode( $response['body'], true );
												if ( is_array( $subresults ) ) {
													foreach ( $subresults as $subresult ) {
														if ( isset( $subresult['date'] ) && ( isset( $subresult['kind'] ) ) ) {
															$event = '';
															if ( $subresult['kind'] === 'OPEN' ) {
																$event = 'open';
															} else if ( $subresult['kind'] === 'HIT' ) {
																$event = 'click';
															} else if ( $subresult['kind'] === 'UNSUBSCRIBE' ) {
																$event = 'unsubscribe';
															}
															if ( $event !== '' ) {
																$json['timeline'][] = array(
																	'date'  => $this->core->get_mysql_date( $subresult['date'] ),
																	'event' => $event
																);
															}
															if ( count( $json ) === 500 ) {
																break;
															}
														}
													}
												}
											}
										}
									}
								} else {
									if ( isset( $result['opens'] ) && isset( $result['hits'] ) && isset( $result['sendIds'] ) ) {
										if ( is_array( $result['sendIds'] ) ) {
											$json             = array(
												'sends'  => count( $result['sendIds'] ),
												'clicks' => $result['hits'],
												'opens'  => $result['opens']
											);
											$json['timeline'] = array();
											$url              = $this->core->get_jackmail_url_analytics() . 'recipient/timeline';
											$timeout          = 30;
											$response         = $this->core->remote_post_retry( $url, $headers, $body, $timeout );
											if ( is_array( $response ) ) {
												if ( isset( $response['body'] ) ) {
													$subresults = json_decode( $response['body'], true );
													if ( is_array( $subresults ) ) {
														foreach ( $subresults as $subresult ) {
															if ( isset( $subresult['event'] ) && isset( $subresult['date'] ) ) {
																$event = $subresult['event'];
																if ( $event !== 'SUCCESS' ) {
																	$event = $this->core->str_to_lower( $event );
																	if ( $event === 'hit' ) {
																		$event = 'click';
																	}
																	$json['timeline'][] = array(
																		'date'  => $this->core->get_mysql_date( $subresult['date'] ),
																		'event' => $event
																	);
																	if ( count( $json ) === 500 ) {
																		break;
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
								break;
							}
						}
					}
				}
			}
		}
		wp_send_json( $json );
		die;
	}

	public function add_header_column_callback() {
		$this->core->check_auth();
		if ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) {
			if ( isset( $_POST['id_list'] ) ) {
				$id_list     = $this->core->request_text_data( $_POST['id_list'] );
				$header_data = $this->get_header_list_params( $id_list );
			} else {
				$id_campaign = $this->core->request_text_data( $_POST['id_campaign'] );
				$header_data = $this->get_header_campaign_params( $id_campaign );
				$this->check_if_campaign_is_draft( $id_campaign );
			}
			$table_lists         = $header_data ['table_lists'];
			$table_list_contacts = $header_data ['table_lists_contacts'];
			$where_id            = $header_data ['where_id'];
			$fields              = $header_data['fields'];
			$fields_array        = $this->core->explode_fields( $fields );
			$nb_fields           = count( $fields_array );
			$field               = 'field' . ( $nb_fields + 1 );
			$columns             = $this->core->get_table_columns( $table_list_contacts, false );
			if ( ! in_array( $field, $columns ) ) {
				$alter_return = $this->core->create_list_field( $table_list_contacts, $field );
				if ( $alter_return !== false ) {
					$fields          = $this->core->explode_fields( $fields );
					$fields_name_try = '( ' . ( $nb_fields + 1 ) . ' )';
					$i               = 1;
					while ( in_array( $fields_name_try, $fields_array ) ) {
						$fields_name_try = '( ' . ( $nb_fields + 1 ) . '-' . $i . ' )';
						$i ++;
					}
					$fields[] = $fields_name_try;
					$fields   = $this->core->implode_fields( $fields );
					$this->updated_list_contact_or_campaign_list_contact_fields( $table_lists, $where_id, $fields );
					wp_send_json_success();
				}
			}
		}
		wp_send_json_error();
		die;
	}

	public function edit_header_column_callback() {
		$this->core->check_auth();
		$json_error = array(
			'message' => '',
			'success' => false
		);
		if ( ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) && isset( $_POST['field_id'] ) && isset( $_POST['field'] ) ) {
			$field_id = $this->core->request_text_data( $_POST['field_id'] );
			$field    = $this->core->request_text_data( $_POST['field'] );
			if ( isset( $_POST['id_list'] ) ) {
				$id_list     = $this->core->request_text_data( $_POST['id_list'] );
				$header_data = $this->get_header_list_params( $id_list );
			} else {
				$id_campaign = $this->core->request_text_data( $_POST['id_campaign'] );
				$header_data = $this->get_header_campaign_params( $id_campaign );
				$this->check_if_campaign_is_draft( $id_campaign );
			}
			if ( $json_error['message'] === '' ) {
				$table_lists  = $header_data ['table_lists'];
				$where_id     = $header_data ['where_id'];
				$fields       = $header_data['fields'];
				$fields_array = $this->core->explode_fields( $fields );
				if ( ( ! in_array( $field, $fields_array ) || $fields_array[ $field_id - 1 ] === $field ) && $field !== 'EMAIL' && $field !== 'BLACKLIST' ) {
					$fields_array[ $field_id - 1 ] = $field;
					$fields                        = $this->core->implode_fields( $fields_array );
					$this->updated_list_contact_or_campaign_list_contact_fields( $table_lists, $where_id, $fields );
					wp_send_json_success();
				} else {
					$json_error ['message'] = __( 'The column name has been used already', 'jackmail-newsletters' );
				}
			}
		}
		wp_send_json( $json_error );
		die;
	}

	public function delete_header_column_callback() {
		$this->core->check_auth();
		if ( ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) && isset( $_POST['field_id'] ) ) {
			$field_id = $this->core->request_text_data( $_POST['field_id'] );
			if ( isset( $_POST['id_list'] ) ) {
				$id_list     = $this->core->request_text_data( $_POST['id_list'] );
				$header_data = $this->get_header_list_params( $id_list );
			} else {
				$id_campaign = $this->core->request_text_data( $_POST['id_campaign'] );
				$header_data = $this->get_header_campaign_params( $id_campaign );
				$this->check_if_campaign_is_draft( $id_campaign );
			}
			$table_lists         = $header_data ['table_lists'];
			$table_list_contacts = $header_data ['table_lists_contacts'];
			$where_id            = $header_data ['where_id'];
			$fields_array        = $this->core->explode_fields( $header_data['fields'] );
			$fields              = array();
			foreach ( $fields_array as $key => $field ) {
				$i = strval( $key + 1 );
				if ( $i !== $field_id ) {
					$fields[] = $field;
				}
			}
			$fields       = $this->core->implode_fields( $fields );
			$nb_fields    = count( $fields_array );
			$alter_return = $this->core->delete_list_field( $table_list_contacts, $field_id, $nb_fields );
			if ( $alter_return !== false ) {
				$this->updated_list_contact_or_campaign_list_contact_fields( $table_lists, $where_id, $fields );
				wp_send_json_success();
			}
		}
		wp_send_json_error();
		die;
	}

	public function delete_contacts_selection_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) && isset( $_POST['search'] ) && isset( $_POST['targeting_rules'] ) && isset( $_POST['contacts_selection'] ) && isset( $_POST['contacts_selection_type'] ) ) {
			$search                  = $this->core->request_text_data( $_POST['search'] );
			$targeting_rules         = $this->core->request_text_data( $_POST['targeting_rules'] );
			$contacts_selection      = $this->core->request_text_data( $_POST['contacts_selection'] );
			$contacts_selection_type = $this->core->request_text_data( $_POST['contacts_selection_type'] );
			if ( isset( $_POST['id_list'] ) ) {
				$id                  = $this->core->request_text_data( $_POST['id_list'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id}";
				$type                = 'list';
			} else {
				$id                  = $this->core->request_text_data( $_POST['id_campaign'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id}";
				$this->check_if_campaign_is_draft( $id );
				$type = 'campaign';
			}
			$contacts_sql = $this->get_list_or_get_campaign_contacts_sql( $table_list_contacts, 'ALL_FILTERED', '', '', '', $search, $targeting_rules, $contacts_selection, $contacts_selection_type );
			if ( isset( $contacts_sql['sql_delete_contacts'] ) && isset( $contacts_sql['sql_values'] ) ) {
				$sql_delete_contacts = $contacts_sql['sql_delete_contacts'];
				$sql_values          = $contacts_sql['sql_values'];
				if ( count( $sql_values ) === 0 ) {
					$delete = $wpdb->query( $sql_delete_contacts );
				} else {
					$delete = $wpdb->query( $wpdb->prepare( $sql_delete_contacts, $sql_values ) );
				}
				if ( $delete !== false ) {
					if ( $type === 'list' ) {
						$this->core->updated_list_contact( $id );
						do_action( 'jackmail_list_contacts_selection_deleted' );
					} else {
						$this->core->updated_campaign_list_contact( $id );
					}
					wp_send_json_success();
				}
			}
		}
		wp_send_json_error();
		die;
	}

	public function delete_all_contacts_callback() {
		$this->core->check_auth();
		global $wpdb;
		if ( isset( $_POST['id_list'] ) || isset( $_POST['id_campaign'] ) ) {
			if ( isset( $_POST['id_list'] ) ) {
				$id                  = $this->core->request_text_data( $_POST['id_list'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id}";
				$type                = 'list';
			} else {
				$id                  = $this->core->request_text_data( $_POST['id_campaign'] );
				$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id}";
				$this->check_if_campaign_is_draft( $id );
				$type = 'campaign';
			}
			$wpdb->query( "TRUNCATE TABLE `{$table_list_contacts}`" );
			if ( $type === 'list' ) {
				$this->core->updated_list_contact( $id );
				do_action( 'jackmail_list_contacts_all_deleted' );
			} else {
				$this->core->updated_campaign_list_contact( $id );
			}
			wp_send_json_success();
		}
		wp_send_json_error();
		die;
	}

	private function check_if_campaign_is_draft( $id_campaign ) {
		global $wpdb;
		$sql = "
		SELECT COUNT( * )
		FROM `{$wpdb->prefix}jackmail_campaigns`
		WHERE `id` = %s
		AND `status` = 'DRAFT'";
		if ( $wpdb->get_var( $wpdb->prepare( $sql, $id_campaign ) ) !== '1' ) {
			$json_error = array(
				'success' => false,
				'message' => __( 'You can\'t edit the campaign.', 'jackmail-newsletters' )
			);
			wp_send_json( $json_error );
		}
	}

	private function updated_list_contact_or_campaign_list_contact_time( $table_lists, $where_id ) {
		return $this->core->updated_list_contact_or_campaign_list_contact( $table_lists, $where_id, array() );
	}

	private function updated_list_contact_or_campaign_list_contact_fields( $table_lists, $where_id, $fields ) {
		$update = array(
			'fields' => $fields
		);
		return $this->core->updated_list_contact_or_campaign_list_contact( $table_lists, $where_id, $update );
	}

	private function get_header_campaign_params( $id_campaign ) {
		global $wpdb;
		$table_lists         = "{$wpdb->prefix}jackmail_campaigns";
		$table_list_contacts = "{$wpdb->prefix}jackmail_campaigns_lists_contacts_{$id_campaign}";
		$sql                 = "
		SELECT `fields`
		FROM `{$table_lists}`
		WHERE `id` = %s";
		$list                = $wpdb->get_row( $wpdb->prepare( $sql, $id_campaign ) );
		$fields              = '';
		if ( isset( $list->fields ) ) {
			$fields = $list->fields;
		}
		return array(
			'id'                   => $id_campaign,
			'table_lists'          => $table_lists,
			'table_lists_contacts' => $table_list_contacts,
			'where_id'             => $id_campaign,
			'fields'               => $fields
		);
	}

	private function get_header_list_params( $id_list ) {
		global $wpdb;
		$table_lists         = "{$wpdb->prefix}jackmail_lists";
		$table_list_contacts = "{$wpdb->prefix}jackmail_lists_contacts_{$id_list}";
		$sql                 = "
		SELECT `fields`
		FROM `{$table_lists}`
		WHERE `id` = %s";
		$list                = $wpdb->get_row( $wpdb->prepare( $sql, $id_list ) );
		$fields              = '';
		if ( isset( $list->fields ) ) {
			$fields = $list->fields;
		}
		return array(
			'id'                   => $id_list,
			'table_lists'          => $table_lists,
			'table_lists_contacts' => $table_list_contacts,
			'where_id'             => $id_list,
			'fields'               => $fields
		);
	}

}